﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Windows.Forms;
using Administración.ServiceReference1;

namespace Administración
{
    public partial class UIAltaCarta : Form
    {
        private ServiceReference1.Service1Client client; //variable per accedir al servei WCF
        private List<Plato> platosDisponibles;//llista de plats de la BBDD
        private List<Plato> platosAsignados;//llista de plats de la BBDD
        private List<int> platosSumar;//llista de plats que s'afegiran
        private Carta oldCarta;

        public UIAltaCarta(Carta carta)
        {
            InitializeComponent();
            client = new ServiceReference1.Service1Client();//nova instancia del servei WCF
            platosSumar = new List<int>();
            this.oldCarta = carta;
        }

        private void UIAltaCarta_Load(object sender, EventArgs e)
        {
            if (oldCarta == null)
            {
                this.dateTimePicker1.Value = DateTime.Now;
                textBox_nombre.Focus();
                //es carreguen tots els plats
                cargarPlatos();
            }
            else
            {
                this.textBox_nombre.Text = this.oldCarta.Nombre;
                this.dateTimePicker1.Value = this.oldCarta.Fecha;
                cargarPlatosDisponibles(oldCarta.Id);
                cargarPlatosAsignados(oldCarta.Id);
            }
        }

        

        //mètode per carregar els plats de la BBDD a la llista de plats disponibles
        private void cargarPlatos()
        {
            try
            {
                //netejem la llista per si te dades
                dataGridView_plDisponibles.ClearSelection();
                //recuperem els plats de la BBDD
                platosDisponibles = client.ListarPlatos().ToList();
                for (int i = 0; i < platosDisponibles.Count(); i++)
                {//Carreguem la llista de plats                     
                    dataGridView_plDisponibles.Rows.Add();
                    dataGridView_plDisponibles.Rows[i].Cells[0].Value = platosDisponibles.ElementAt(i).Id;
                    dataGridView_plDisponibles.Rows[i].Cells[1].Value = platosDisponibles.ElementAt(i).Idioma1;
                }
            }
            catch (NullReferenceException ex)
            {
                MessageBox.Show(ex.Message, "Platos Disponibles: " + ex.Source, MessageBoxButtons.OK, MessageBoxIcon.Error);
            }
            catch (SystemException ex1)
            {
                MessageBox.Show(ex1.Message, "Platos Disponibles: " + ex1.Source, MessageBoxButtons.OK, MessageBoxIcon.Error);
            }
            catch (Exception ex2)
            {
                MessageBox.Show(ex2.Message, "Platos Disponibles: " + ex2.Source, MessageBoxButtons.OK, MessageBoxIcon.Error);
            }
        }

        //mètode per carregar els plats de la BBDD a la llista de plats disponibles quan la carta ja te plats
        private void cargarPlatosDisponibles(int idCarta)
        {
            try
            {
                //netejem la llista per si te dades
                dataGridView_plAsignados.ClearSelection();
                //recuperem els plats de la BBDD
                platosDisponibles = client.ListarPlDisponiblesCarta(idCarta).ToList();
                for (int i = 0; i < platosDisponibles.Count(); i++)
                {//Carreguem la llista de plats                     
                    dataGridView_plDisponibles.Rows.Add();
                    dataGridView_plDisponibles.Rows[i].Cells[0].Value = platosDisponibles.ElementAt(i).Id;
                    dataGridView_plDisponibles.Rows[i].Cells[1].Value = platosDisponibles.ElementAt(i).Idioma1;
                }
            }
            catch (NullReferenceException ex)
            {
                MessageBox.Show(ex.Message, "Platos Disponibles: " + ex.Source, MessageBoxButtons.OK, MessageBoxIcon.Error);
            }
            catch (SystemException ex1)
            {
                MessageBox.Show(ex1.Message, "Platos Disponibles: " + ex1.Source, MessageBoxButtons.OK, MessageBoxIcon.Error);
            }
            catch (Exception ex2)
            {
                MessageBox.Show(ex2.Message, "Platos Disponibles: " + ex2.Source, MessageBoxButtons.OK, MessageBoxIcon.Error);
            }
        }

        //mètode per carregar els plats de la BBDD a la llista de plats assignats quan la carta ja te plats
        private void cargarPlatosAsignados(int idCarta)
        {
            try
            {
                //netejem la llista per si te dades
                dataGridView_plAsignados.ClearSelection();
                //recuperem els plats de la BBDD
                platosAsignados = client.ListarPlAsignadosCarta(idCarta).ToList();
                for (int i = 0; i < platosAsignados.Count(); i++)
                {//Carreguem la llista de plats                     
                    dataGridView_plAsignados.Rows.Add();
                    dataGridView_plAsignados.Rows[i].Cells[0].Value = platosAsignados.ElementAt(i).Id;
                    dataGridView_plAsignados.Rows[i].Cells[1].Value = platosAsignados.ElementAt(i).Idioma1;
                }
            }
            catch (NullReferenceException ex)
            {
                MessageBox.Show(ex.Message, "Platos Asignados: " + ex.Source, MessageBoxButtons.OK, MessageBoxIcon.Error);
            }
            catch (SystemException ex1)
            {
                MessageBox.Show(ex1.Message, "Platos Asignados: " + ex1.Source, MessageBoxButtons.OK, MessageBoxIcon.Error);
            }
            catch (Exception ex2)
            {
                MessageBox.Show(ex2.Message, "Platos Asignados: " + ex2.Source, MessageBoxButtons.OK, MessageBoxIcon.Error);
            }
        }

        

        private void button_SumarPlato_Click(object sender, EventArgs e)
        {

            if (this.dataGridView_plDisponibles.Rows.Count > 0) //nº de files del 1r datagrid
            {

                DataGridViewRow row = this.dataGridView_plDisponibles.CurrentRow; //fila seleccionada
                int celda1 = Convert.ToInt32(row.Cells[0].Value);
                string celda2 = Convert.ToString(row.Cells[1].Value);

                //passem el plat al 2nd datagrid  
                int i = this.dataGridView_plAsignados.Rows.Count;
                dataGridView_plAsignados.Rows.Add();
                dataGridView_plAsignados.Rows[i].Cells[0].Value = celda1;
                dataGridView_plAsignados.Rows[i].Cells[1].Value = celda2;
                //treiem el plat del 1r datagrid
                dataGridView_plDisponibles.Rows.Remove(row);
            }
            else //no queden més files per traspassar
            {
                MessageBox.Show("No hay platos para adicionar", "Añadir Platos", MessageBoxButtons.OK, MessageBoxIcon.Warning);
            }
        }

       
        private void button_SacarPlato_Click(object sender, EventArgs e)
        {
            if (this.dataGridView_plAsignados.Rows.Count > 0) //nº de files del 2nd datagrid
            {

                DataGridViewRow row = this.dataGridView_plAsignados.CurrentRow; //fila seleccionada
                int celda1 = Convert.ToInt32(row.Cells[0].Value);
                string celda2 = Convert.ToString(row.Cells[1].Value);

                //passem el plat al 2nd datagrid  
                int i = this.dataGridView_plDisponibles.Rows.Count;
                dataGridView_plDisponibles.Rows.Add();
                dataGridView_plDisponibles.Rows[i].Cells[0].Value = celda1;
                dataGridView_plDisponibles.Rows[i].Cells[1].Value = celda2;
                //treiem el plat del 1r datagrid
                dataGridView_plAsignados.Rows.Remove(row);
            }
            else //no queden més files per traspassar
            {
                MessageBox.Show("No hay platos para restar", "Añadir Platos", MessageBoxButtons.OK, MessageBoxIcon.Warning);
            }

        }

        private void button_guardar_Click(object sender, EventArgs e)
        {
            if (oldCarta == null)
            { //nova carta
                this.añadirCarta();
            }
            else {//modificarcarta 
                this.modificarCarta();
            }

        }

        private void button_cancelar_Click(object sender, EventArgs e)
        {
            this.Dispose();
        }

        

        private void button_seleccionar_Click(object sender, EventArgs e)
        {
            this.panel1.Enabled = false;
            this.panel2.Visible = true;
        }

        private void button_volver_Click(object sender, EventArgs e)
        {
            this.panel1.Enabled = true;
            this.panel2.Visible = false;
        }

        //mètode intern per afegir una carta i els seus plats
        private void añadirCarta()
        {
            try
            {
                for (int i = 0; i < this.dataGridView_plAsignados.Rows.Count; i++)
                {//guardem la llista de idPlat seleccionats per la carta  
                    this.platosSumar.Add(Convert.ToInt32(dataGridView_plAsignados.Rows[i].Cells[0].Value));
                }

                DateTime dataSelected = DateTime.Parse(this.dateTimePicker1.Text);
                //afegim la nova carta i obtenim el valor del nou identificador de la carta
                int idNovaCarta = client.AltaCarta(dataSelected, this.textBox_nombre.Text, false);

                //afegim els plats assignats a la nova carta
                for (int j = 0; j < this.platosSumar.Count; j++)
                {
                    int idPlatoSumar = this.platosSumar.ElementAt(j);
                    client.AñadirPlatoCarta(idNovaCarta, idPlatoSumar);
                }
                //tanquem la finestra
                this.Dispose();
            }
            catch (NullReferenceException ex)
            {
                MessageBox.Show(ex.Message, "Nueva Carta: " + ex.Source, MessageBoxButtons.OK, MessageBoxIcon.Error);
            }
            catch (SystemException ex1)
            {
                MessageBox.Show(ex1.Message, "Nueva Carta: " + ex1.Source, MessageBoxButtons.OK, MessageBoxIcon.Error);
            }
            catch (Exception ex2)
            {
                MessageBox.Show(ex2.Message, "Nueva Carta: " + ex2.Source, MessageBoxButtons.OK, MessageBoxIcon.Error);
            }
        }

        //mètode intern per modificar una carta i els seus plats
        private void modificarCarta()
        {
            try
            {
                //eliminem els plats de la carta
                client.EliminarPlatosCarta(oldCarta.Id);

                for (int i = 0; i < this.dataGridView_plAsignados.Rows.Count; i++)
                {//guardem la llista de idPlat seleccionats per la carta  
                    this.platosSumar.Add(Convert.ToInt32(dataGridView_plAsignados.Rows[i].Cells[0].Value));
                }
                DateTime dataSelected = DateTime.Parse(this.dateTimePicker1.Text);
                //modifiquem la carta 
                client.ModificarCarta(oldCarta.Id, dataSelected, this.textBox_nombre.Text);

                //afegim els plats assignats a la carta
                for (int j = 0; j < this.platosSumar.Count; j++)
                {
                    int idPlatoSumar = this.platosSumar.ElementAt(j);
                    client.AñadirPlatoCarta(oldCarta.Id, idPlatoSumar);
                }
                //tanquem la finestra
                this.Dispose();
            }
            catch (NullReferenceException ex)
            {
                MessageBox.Show(ex.Message, "Modificar Carta: " + ex.Source, MessageBoxButtons.OK, MessageBoxIcon.Error);
            }
            catch (SystemException ex1)
            {
                MessageBox.Show(ex1.Message, "Modificar Carta: " + ex1.Source, MessageBoxButtons.OK, MessageBoxIcon.Error);
            }
            catch (Exception ex2)
            {
                MessageBox.Show(ex2.Message, "Modificar Carta: " + ex2.Source, MessageBoxButtons.OK, MessageBoxIcon.Error);
            }
        }

    }
}
