﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Windows.Forms;
using Administración.ServiceReference1;

namespace Administración
{
    public partial class UIAltaMenu : Form
    {
        private ServiceReference1.Service1Client client; //variable per accedir al servei WCF
        private List<Plato> platosDisponibles;//llista de plats de la BBDD
        private List<Plato> platosAsignados;//llista de plats de la BBDD
        private List<int> platosSumar;//llista de plats que s'afegiran
        private List<int> tiposSumar;//llista de tipus que s'afegiran
        private List<Tipo> tipos;
        private Administración.ServiceReference1.Menu oldMenu;


        public UIAltaMenu(Administración.ServiceReference1.Menu menu)
        {
            InitializeComponent();
            client = new ServiceReference1.Service1Client();//nova instancia del servei WCF
            platosSumar = new List<int>();
            tiposSumar = new List<int>();
            this.oldMenu = menu;
        }

        private void UIAltaMenu_Load(object sender, EventArgs e)
        {
            if (oldMenu == null)
            {
                this.dateTimePicker1.Value = DateTime.Now;
                textBox_nombre.Focus();
                cargarListaTipos();
                //es carreguen tots els plats
                cargarPlatos();
            }
            else
            {
                cargarListaTipos();
                this.textBox_nombre.Text = this.oldMenu.Nombre;
                this.dateTimePicker1.Value = this.oldMenu.Fecha;
                this.textBox_comensales.Text = this.oldMenu.MinComensales.ToString();
                this.textBox_precio.Text = this.oldMenu.Precio.ToString();
                this.checkBox_bebida.Checked = Convert.ToBoolean(this.oldMenu.Bebida);
                this.checkBox_iva.Checked = Convert.ToBoolean(this.oldMenu.Iva);
                cargarPlatosDisponibles(oldMenu.Id);
                cargarPlatosAsignados(oldMenu.Id);
            }
        }

        //mètode per carregar els plats de la BBDD a la llista de plats disponibles
        private void cargarPlatos()
        {
            try
            {
                //netejem la llista per si te dades
                dataGridView_plDisponibles.ClearSelection();
                //recuperem els plats de la BBDD
                platosDisponibles = client.ListarPlatos().ToList();
                for (int i = 0; i < platosDisponibles.Count(); i++)
                {//Carreguem la llista de plats                     
                    dataGridView_plDisponibles.Rows.Add();
                    dataGridView_plDisponibles.Rows[i].Cells[0].Value = platosDisponibles.ElementAt(i).Id;
                    dataGridView_plDisponibles.Rows[i].Cells[1].Value = platosDisponibles.ElementAt(i).Idioma1;
                }
            }
            catch (NullReferenceException ex)
            {
                MessageBox.Show(ex.Message, "Platos Disponibles: " + ex.Source, MessageBoxButtons.OK, MessageBoxIcon.Error);
            }
            catch (SystemException ex1)
            {
                MessageBox.Show(ex1.Message, "Platos Disponibles: " + ex1.Source, MessageBoxButtons.OK, MessageBoxIcon.Error);
            }
            catch (Exception ex2)
            {
                MessageBox.Show(ex2.Message, "Platos Disponibles: " + ex2.Source, MessageBoxButtons.OK, MessageBoxIcon.Error);
            }
        }


        //mètode per carregar els plats de la BBDD a la llista de plats disponibles quan el menu ja te plats
        private void cargarPlatosDisponibles(int idMenu)
        {
            try
            {
                //netejem la llista per si te dades
                dataGridView_plDisponibles.ClearSelection();
                //recuperem els plats de la BBDD
                platosDisponibles = client.ListarPlDisponiblesMenu(idMenu).ToList();
                for (int i = 0; i < platosDisponibles.Count(); i++)
                {//Carreguem la llista de plats                     
                    dataGridView_plDisponibles.Rows.Add();
                    dataGridView_plDisponibles.Rows[i].Cells[0].Value = platosDisponibles.ElementAt(i).Id;
                    dataGridView_plDisponibles.Rows[i].Cells[1].Value = platosDisponibles.ElementAt(i).Idioma1;
                }
            }
            catch (NullReferenceException ex)
            {
                MessageBox.Show(ex.Message, "Platos Disponibles: " + ex.Source, MessageBoxButtons.OK, MessageBoxIcon.Error);
            }
            catch (SystemException ex1)
            {
                MessageBox.Show(ex1.Message, "Platos Disponibles: " + ex1.Source, MessageBoxButtons.OK, MessageBoxIcon.Error);
            }
            catch (Exception ex2)
            {
                MessageBox.Show(ex2.Message, "Platos Disponibles: " + ex2.Source, MessageBoxButtons.OK, MessageBoxIcon.Error);
            }
        }


        //mètode per carregar els plats de la BBDD a la llista de plats assignats quan el menu ja te plats
        private void cargarPlatosAsignados(int idMenu)
        {
            try
            {
                //netejem la llista per si te dades
                dataGridView_plAsignados.ClearSelection();
                //recuperem els plats de la BBDD
                platosAsignados = client.ListarPlAsignadosMenu(idMenu).ToList();
                for (int i = 0; i < platosAsignados.Count(); i++)
                {//Carreguem la llista de plats                     
                    dataGridView_plAsignados.Rows.Add();
                    dataGridView_plAsignados.Rows[i].Cells[0].Value = platosAsignados.ElementAt(i).Id;
                    dataGridView_plAsignados.Rows[i].Cells[1].Value = platosAsignados.ElementAt(i).Idioma1;
                    int tipo = client.buscarTipoPlatoMenu(idMenu, platosAsignados.ElementAt(i).Id);
                    dataGridView_plAsignados.Rows[i].Cells[3].Value = tipo;
                    dataGridView_plAsignados.Rows[i].Cells[2].Value = client.nombreTipo(tipo);
                }
            }
            catch (NullReferenceException ex)
            {
                MessageBox.Show(ex.Message, "Platos Disponibles: " + ex.Source, MessageBoxButtons.OK, MessageBoxIcon.Error);
            }
            catch (SystemException ex1)
            {
                MessageBox.Show(ex1.Message, "Platos Disponibles: " + ex1.Source, MessageBoxButtons.OK, MessageBoxIcon.Error);
            }
            catch (Exception ex2)
            {
                MessageBox.Show(ex2.Message, "Platos Disponibles: " + ex2.Source, MessageBoxButtons.OK, MessageBoxIcon.Error);
            }
        }


        private void cargarListaTipos()
        {
            try
            {
                //netejem la llista per si te dades
                this.comboBox_tipos.Items.Clear();
                //recuperem els tipus de la BBDD
                tipos = client.ListarTipos().ToList();
                for (int i = 0; i < tipos.Count(); i++)
                {//Carreguem la llista de tipus
                    this.comboBox_tipos.Items.Add(tipos.ElementAt(i).Tipo1);
                }
            }
            catch (NullReferenceException ex)
            {
                MessageBox.Show(ex.Message, "Tipos: " + ex.Source, MessageBoxButtons.OK, MessageBoxIcon.Error);
            }
            catch (SystemException ex1)
            {
                MessageBox.Show(ex1.Message, "Tipos: " + ex1.Source, MessageBoxButtons.OK, MessageBoxIcon.Error);
            }
            catch (Exception ex2)
            {
                MessageBox.Show(ex2.Message, "Tipos: " + ex2.Source, MessageBoxButtons.OK, MessageBoxIcon.Error);
            }
        }

        private void button_cancelar_Click(object sender, EventArgs e)
        {
            this.Dispose();
        }

       

        private void button_SumarPlato_Click(object sender, EventArgs e)
        {
            if (this.dataGridView_plDisponibles.Rows.Count > 0) //nº de files del 1r datagrid
            {
                
                    //recuperem la fila seleccionada
                DataGridViewRow row = this.dataGridView_plDisponibles.CurrentRow; 
                int celda1 = Convert.ToInt32(row.Cells[0].Value);
                string celda2 = Convert.ToString(row.Cells[1].Value);
                //recuperem el tipus seleccionat al combobox
                int item = comboBox_tipos.SelectedIndex;
                int celda4 = tipos.ElementAt(item).Id;
                string celda3 = tipos.ElementAt(item).Tipo1;
                //passem el plat al 2nd datagrid  
                int i = this.dataGridView_plAsignados.Rows.Count;
                dataGridView_plAsignados.Rows.Add();
                dataGridView_plAsignados.Rows[i].Cells[0].Value = celda1;
                dataGridView_plAsignados.Rows[i].Cells[1].Value = celda2;
                dataGridView_plAsignados.Rows[i].Cells[2].Value = celda3;
                dataGridView_plAsignados.Rows[i].Cells[3].Value = celda4;
                //treiem el plat del 1r datagrid
                dataGridView_plDisponibles.Rows.Remove(row);
                
                
            }
            else //no queden més files per traspassar
            {
                MessageBox.Show("No hay platos para adicionar", "Añadir Platos", MessageBoxButtons.OK, MessageBoxIcon.Warning);
            }

        }

        private void button_SacarPlato_Click(object sender, EventArgs e)
        {
            if (this.dataGridView_plAsignados.Rows.Count > 0) //nº de files del 2nd datagrid
            {

                DataGridViewRow row = this.dataGridView_plAsignados.CurrentRow; //fila seleccionada
                int celda1 = Convert.ToInt32(row.Cells[0].Value);
                string celda2 = Convert.ToString(row.Cells[1].Value);

                //passem el plat al 2nd datagrid  
                int i = this.dataGridView_plDisponibles.Rows.Count;
                dataGridView_plDisponibles.Rows.Add();
                dataGridView_plDisponibles.Rows[i].Cells[0].Value = celda1;
                dataGridView_plDisponibles.Rows[i].Cells[1].Value = celda2;
                //treiem el plat del 1r datagrid
                dataGridView_plAsignados.Rows.Remove(row);
            }
            else //no queden més files per traspassar
            {
                MessageBox.Show("No hay platos para restar", "Añadir Platos", MessageBoxButtons.OK, MessageBoxIcon.Warning);
            }

        }

        private void comboBox_tipos_SelectedIndexChanged(object sender, EventArgs e)
        {
            this.button_SumarPlato.Enabled = true;
        }

        private void button_guardar_Click(object sender, EventArgs e)
        {
            if (oldMenu == null)
            { //nou menu
                this.AñadirMenu();
            }
            else
            {//modificar menu 
                this.ModificarMenu();
            }
        }

        private void button_seleccion_Click(object sender, EventArgs e)
        {
            if (textBox_precio.Text.Length == 0)//validem que s'ha entrat un preu
            {
                MessageBox.Show("Falta introducir un precio para el menu", "Menu", MessageBoxButtons.OK, MessageBoxIcon.Warning);
            }
            else
            {
                this.panel1.Enabled = false;
                this.panel2.Visible = true;
            }
        }

        private void button_volver_Click(object sender, EventArgs e)
        {
            this.panel1.Enabled = true;
            this.panel2.Visible = false;
        }

        //mètode intern per afegir un nou menu i els seus plats
        private void AñadirMenu()
        {
            try
            {
                
                for (int i = 0; i < this.dataGridView_plAsignados.Rows.Count; i++)
                {//guardem la llista de idPlat seleccionats per la carta 
                    this.platosSumar.Add(Convert.ToInt32(dataGridView_plAsignados.Rows[i].Cells[0].Value));
                    this.tiposSumar.Add(Convert.ToInt32(dataGridView_plAsignados.Rows[i].Cells[3].Value));
                    
                }

                DateTime dataSelected = DateTime.Parse(this.dateTimePicker1.Text);
                //afegim el nou menu i obtenim el valor del nou identificador del menu
                int idNouMenu = client.AltaMenu(dataSelected, this.textBox_nombre.Text, float.Parse(this.textBox_precio.Text), int.Parse(this.textBox_comensales.Text),
                    this.checkBox_bebida.Checked, this.checkBox_iva.Checked, false);
                

                //afegim els plats assignats al nou menu
                for (int j = 0; j < this.platosSumar.Count; j++)
                {
                    int idPlatoSumar = this.platosSumar.ElementAt(j);
                    int idTipoSumar = this.tiposSumar.ElementAt(j);
                    client.AñadirPlatoMenu(idNouMenu, idPlatoSumar, idTipoSumar);
                    
                }
                //tanquem la finestra
                this.Dispose();
            }
            catch (NullReferenceException ex)
            {
                MessageBox.Show(ex.Message, "Nuevo Menu 1: " + ex.Source, MessageBoxButtons.OK, MessageBoxIcon.Error);
            }
            catch (SystemException ex1)
            {
                MessageBox.Show(ex1.Message, "Nuevo Menu 2: " + ex1.Source, MessageBoxButtons.OK, MessageBoxIcon.Error);
            }
            catch (Exception ex2)
            {
                MessageBox.Show(ex2.Message, "Nuevo Menu 3: " + ex2.Source, MessageBoxButtons.OK, MessageBoxIcon.Error);
            }
        }

        private void ModificarMenu()
        {
            try
            {
                //eliminem els plats del menu
                client.EliminarPlatosMenu(oldMenu.Id);

                for (int i = 0; i < this.dataGridView_plAsignados.Rows.Count; i++)
                {//guardem la llista de idPlat seleccionats per la carta 
                    this.platosSumar.Add(Convert.ToInt32(dataGridView_plAsignados.Rows[i].Cells[0].Value));
                    this.tiposSumar.Add(Convert.ToInt32(dataGridView_plAsignados.Rows[i].Cells[3].Value));
                }

                DateTime dataSelected = DateTime.Parse(this.dateTimePicker1.Text);
                //modifiquem el menu 
               client.ModificarMenu(oldMenu.Id, dataSelected, this.textBox_nombre.Text, float.Parse(this.textBox_precio.Text), 
                   int.Parse(this.textBox_comensales.Text), this.checkBox_bebida.Checked, this.checkBox_iva.Checked);

                //afegim els plats assignats al nou menu
                for (int j = 0; j < this.platosSumar.Count; j++)
                {
                    int idPlatoSumar = this.platosSumar.ElementAt(j);
                    int idTipoSumar = this.tiposSumar.ElementAt(j);
                    client.AñadirPlatoMenu(oldMenu.Id, idPlatoSumar, idTipoSumar);
                }
                //tanquem la finestra
                this.Dispose();
            }
            catch (NullReferenceException ex)
            {
                MessageBox.Show(ex.Message, "Nuevo Menu: " + ex.Source, MessageBoxButtons.OK, MessageBoxIcon.Error);
            }
            catch (SystemException ex1)
            {
                MessageBox.Show(ex1.Message, "Nuevo Menu: " + ex1.Source, MessageBoxButtons.OK, MessageBoxIcon.Error);
            }
            catch (Exception ex2)
            {
                MessageBox.Show(ex2.Message, "Nuevo Menu: " + ex2.Source, MessageBoxButtons.OK, MessageBoxIcon.Error);
            }
        }

        private void button_cerrar_Click(object sender, EventArgs e)
        {
            this.Dispose();
        }

        
    }
}
