﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Windows.Forms;
using Administración.ServiceReference1;

namespace Administración
{
    public partial class UIAltaPlato : Form
    {
        private ServiceReference1.Service1Client client; //variable per accedir al servei WCF
        private List<Seccion> secciones;
        Plato oldPlato;
        String nombreSeccion;

        public UIAltaPlato(Plato oldPlato, List<Seccion> secciones, string nombreSeccion)
        {
            InitializeComponent();
            client = new ServiceReference1.Service1Client();//nova instancia del servei WCF
            this.oldPlato = oldPlato;
            this.secciones = secciones;
            this.nombreSeccion = nombreSeccion;
        }       

        private void button_cancelar_Click(object sender, EventArgs e)
        {
            this.Dispose();
        }

        private void cargarComboSecciones()
        {
            try
            {
                //netejem la llista per si te dades
                comboBox_seccion.Items.Clear();                
                for (int i = 0; i < secciones.Count(); i++)
                {//Carreguem el combo de seccions   
                    comboBox_seccion.Items.Add(secciones.ElementAt(i).nombre);
                }
            }
            catch (NullReferenceException ex)
            {
                MessageBox.Show(ex.Message, "Nuevo Plato: " + ex.Source, MessageBoxButtons.OK, MessageBoxIcon.Error);
            }
            catch (SystemException ex1)
            {
                MessageBox.Show(ex1.Message, "Nuevo Plato: " + ex1.Source, MessageBoxButtons.OK, MessageBoxIcon.Error);
            }
            catch (Exception ex2)
            {
                MessageBox.Show(ex2.Message, "Nuevo Plato: " + ex2.Source, MessageBoxButtons.OK, MessageBoxIcon.Error);
            }
        }

        private void UIAltaPlato_Load(object sender, EventArgs e)
        {
            //carreguem les dades del combo
            cargarComboSecciones();

            if (oldPlato != null)
            {   //carreguem les dades del plat passat com a paràmetre
                this.textBox_nombrePlato1.Text = oldPlato.Idioma1;
                this.textBox_nombrePlato2.Text = oldPlato.Idioma2;
                this.textBox_nombePlato3.Text = oldPlato.Idioma3;
                this.textBox_precio.Text = oldPlato.Precio.ToString();
                this.checkBox_iva.Checked = oldPlato.Iva;
                this.checkBox_disp.Checked = oldPlato.Disponible;
                this.textBox_comensales.Text = oldPlato.MinComensales.ToString();

                //busquem l'element del combobox associat a la secció
                int i = comboBox_seccion.Items.IndexOf(nombreSeccion);
                comboBox_seccion.SelectedIndex = i;
            }
        }

        private void button_guardar_Click(object sender, EventArgs e)
        {
            if (oldPlato == null)
            { //nou plat
                this.añadirPlato();
            }
            else
            {//modificar plat 
                this.modificarPlato();
            }

        }
               


        private void textBox_precio_Leave(object sender, EventArgs e)
        {
            try
            {
                float.Parse(textBox_precio.Text);
            }
            catch
            {
                MessageBox.Show("El precio debe ser un nº mayor que cero", "Error", MessageBoxButtons.OK, MessageBoxIcon.Error);
                textBox_precio.Clear();
                textBox_precio.Focus();
            }
            
        }

        private void textBox_comensales_Leave(object sender, EventArgs e)
        {
            try
            {
                int.Parse(textBox_comensales.Text);
            }
            catch
            {
                textBox_comensales.Text = "0";
                MessageBox.Show("Comensales incorrectos. El valor debe ser un nº", "Error", MessageBoxButtons.OK, MessageBoxIcon.Error);
                textBox_comensales.Focus();
            }
        }

        private void añadirPlato()
        {
            try
            {
                //comprovem que s'han entrat totes les dades necessàries
                if ((textBox_nombrePlato1.Text.Length == 0) || (textBox_precio.Text.Length == 0)
                    || (comboBox_seccion.SelectedIndex < 0))
                {
                    MessageBox.Show("Faltan Datos", "Guardar Plato", MessageBoxButtons.OK, MessageBoxIcon.Warning);
                }
                else
                {
                    //recuperem la secció del combo box
                    int item = comboBox_seccion.SelectedIndex;
                    int secc = secciones.ElementAt(item).Id;
                    //afegim el plat
                    client.AltaPlato(textBox_nombrePlato1.Text, textBox_nombrePlato2.Text, textBox_nombePlato3.Text,
                        float.Parse(textBox_precio.Text), secc, int.Parse(textBox_comensales.Text),
                        checkBox_iva.Checked, checkBox_disp.Checked);

                    //tanquem el formulari
                    this.Dispose();
                }
            }
            catch (NullReferenceException ex)
            {
                MessageBox.Show(ex.Message, "Guardar Plato: " + ex.Source, MessageBoxButtons.OK, MessageBoxIcon.Error);
            }
            catch (SystemException ex1)
            {
                MessageBox.Show(ex1.Message, "Guardar Plato: " + ex1.Source, MessageBoxButtons.OK, MessageBoxIcon.Error);
            }
            catch (Exception ex2)
            {
                MessageBox.Show(ex2.Message, "Guardar Plato: " + ex2.Source, MessageBoxButtons.OK, MessageBoxIcon.Error);
            }
        }

        private void modificarPlato()
        {
            try
            {
                //comprovem que s'han entrat totes les dades necessaries
                if ((textBox_nombrePlato1.Text.Length == 0) || (textBox_precio.Text.Length == 0)
                    || (comboBox_seccion.SelectedIndex < 0))
                {
                    MessageBox.Show("Faltan Datos", "Guardar Plato", MessageBoxButtons.OK, MessageBoxIcon.Warning);
                }
                else
                {
                    //recuperem la secció del combo box
                    int item = comboBox_seccion.SelectedIndex;
                    int secc = secciones.ElementAt(item).Id;

                    //modifiquem el plat
                    client.ModificarPlato(oldPlato.Id,
                        textBox_nombrePlato1.Text, textBox_nombrePlato2.Text, textBox_nombePlato3.Text,
                        float.Parse(textBox_precio.Text), secc, int.Parse(textBox_comensales.Text),
                        checkBox_iva.Checked, checkBox_disp.Checked);

                    //tanquem el formulari
                    this.Dispose();
                }
            }
            catch (NullReferenceException ex)
            {
                MessageBox.Show(ex.Message, "Guardar Plato: " + ex.Source, MessageBoxButtons.OK, MessageBoxIcon.Error);
            }
            catch (SystemException ex1)
            {
                MessageBox.Show(ex1.Message, "Guardar Plato: " + ex1.Source, MessageBoxButtons.OK, MessageBoxIcon.Error);
            }
            catch (Exception ex2)
            {
                MessageBox.Show(ex2.Message, "Guardar Plato: " + ex2.Source, MessageBoxButtons.OK, MessageBoxIcon.Error);
            }
        }

    }
}
