﻿using System;
using System.Resources;
using System.Collections;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Net.Mail;
using System.Text;
using System.Windows.Forms;
using Administración.ServiceReference1;

namespace Administración
{
    public partial class UIAltaReserva : Form
    {
        private ServiceReference1.Service1Client client; //variable per accedir al servei WCF
        private List<Turno> turnos;
        private List<Mesa> mesas;
        private Mesa mesaRes;
        private Reserva oldReserva;
        private int nuevaReserva; 

        public UIAltaReserva(Reserva pReserva)
        {
            InitializeComponent();
            client = new ServiceReference1.Service1Client();//nova instancia del servei WCF
            mesaRes = new Mesa();
            this.oldReserva = pReserva;
        }

        private void UIAltaReserva_Load(object sender, EventArgs e)
        {
            cargarComboTurnos();   //carreguem el combo de turnos

            //carreguem les dades de l'antiga reserva si existeix
            if (oldReserva != null)
            {
                dateTimePicker1.Value = oldReserva.Fecha.Date;
                comboBox_turnos.SelectedText = oldReserva.Turno;
                textBox_nombre.Text = oldReserva.Nombre;
                textBox_mail.Text = oldReserva.Mail;
                textBox_telef.Text = oldReserva.Telefono;
                textBox_comensales.Text = oldReserva.Comensales.ToString();


            }

            //comprovem que la data no es festiva
            bool festivo = client.BuscarFestivo(this.dateTimePicker1.Value.Date);
            if (festivo == true)
            {
                this.panel1.Visible = false;
                this.label_festivo.Visible = true;
            }
            else
            {
                this.panel1.Visible = true;
                this.label_festivo.Visible = false;
            }

            

        }
        private void button_cancelar_Click(object sender, EventArgs e)
        {
            this.Dispose();
        }

        private void button_guardar_Click(object sender, EventArgs e)
        {
            try
            {
                Console.WriteLine("mesa " + mesaRes.numero);
                if (mesaRes.numero>0)
                {
                    if (oldReserva != null)
                    {//modificar reserva
                        client.EliminarReserva(oldReserva.id, oldReserva.Fecha, oldReserva.Mesa, oldReserva.Turno);
                        nuevaReserva = client.AltaReserva(0, textBox_nombre.Text, textBox_telef.Text, textBox_mail.Text,
                    int.Parse(textBox_comensales.Text), "confirmada", "telefónica", this.dateTimePicker1.Value.Date,
                    mesaRes.numero, this.comboBox_turnos.SelectedItem.ToString());
                        MessageBox.Show("Reserva con Nº: " + nuevaReserva, "Nueva Reserva");
                    }
                    else 
                    { //nova reserva
                        nuevaReserva = client.AltaReserva(0, textBox_nombre.Text, textBox_telef.Text, textBox_mail.Text,
                    int.Parse(textBox_comensales.Text), "confirmada", "telefónica", this.dateTimePicker1.Value.Date,
                    mesaRes.numero, this.comboBox_turnos.SelectedItem.ToString());
                        MessageBox.Show("Reserva con Nº: " + nuevaReserva, "Nueva Reserva");
                    }

                    //envio eMail confirmació de la reserva:
                    //recuperem les dades del nostre servidor
                    string smtpEmail = Administración.Properties.Resources.eMail;
                    string smtpPw = Administración.Properties.Resources.password;
                    string smtpServer = Administración.Properties.Resources.ServidorSmtp;

                    //creem el missatge de mail
                    MailMessage correo = new MailMessage();
                    correo.From = new MailAddress(smtpEmail);
                    correo.To.Add(this.textBox_mail.Text);
                    correo.Subject = "Confirmación Reserva, UOC Restaurant";

                    StringBuilder sb = new StringBuilder();
                    sb = sb.Append("Confirmación de Reserva:");
                    sb = sb.Append("\n");
                    sb = sb.Append("Su reserva nº: " + nuevaReserva);
                    sb = sb.Append("\n");
                    sb = sb.Append(this.label_reserva.Text);
                    sb = sb.Append("\n");
                    sb = sb.Append(this.label_mesa.Text);
                    sb = sb.Append("\n" + "\n");
                    sb = sb.Append("Gracias por su reserva");
                    sb = sb.Append("\n");
                    sb = sb.Append("UOC Restaurant");

                    correo.Body = sb.ToString();
                    correo.IsBodyHtml = false;
                    correo.Priority = MailPriority.Normal;
                    SmtpClient smtp = new SmtpClient(smtpServer);
                    smtp.EnableSsl = true;
                    smtp.UseDefaultCredentials = false;
                    smtp.Port = 25;
                    smtp.Credentials = new System.Net.NetworkCredential(smtpEmail, smtpPw);

                    try
                    {//enviem el missatge
                        smtp.Send(correo);
                    }
                    catch (Exception ex)
                    {
                        MessageBox.Show(ex.Message, "Confirmar Reserva: " + ex.Source, MessageBoxButtons.OK, MessageBoxIcon.Error);
                    }



                    this.Dispose();
                }
                else
                {
                    MessageBox.Show("seleccione una mesa", "Nueva Reserva: " , MessageBoxButtons.OK, MessageBoxIcon.Warning);
                }
               
            }
            catch (NullReferenceException ex)
            {
                MessageBox.Show(ex.Message, "Nueva Reserva: " + ex.Source, MessageBoxButtons.OK, MessageBoxIcon.Error);
            }
            catch (SystemException ex1)
            {
                MessageBox.Show(ex1.Message, "Nueva Reserva: " + ex1.Source, MessageBoxButtons.OK, MessageBoxIcon.Error);
            }
            catch (Exception ex2)
            {
                MessageBox.Show(ex2.Message, "Nueva Reserva: " + ex2.Source, MessageBoxButtons.OK, MessageBoxIcon.Error);
            }
            
        }

       
        private void cargarComboTurnos()
        {
            try
            {
                //netejem la llista per si te dades
                this.comboBox_turnos.Items.Clear();
                //recuperem els torns de la BBDD
                turnos = client.ListarTurnos().ToList();
                for (int i = 0; i < turnos.Count(); i++)
                {//Carreguem la llista de taules
                    this.comboBox_turnos.Items.Add(turnos.ElementAt(i).Hora.ToString());
                }
            }
            catch (NullReferenceException ex)
            {
                MessageBox.Show(ex.Message, "Turnos: " + ex.Source, MessageBoxButtons.OK, MessageBoxIcon.Error);
            }
            catch (SystemException ex1)
            {
                MessageBox.Show(ex1.Message, "Turnos: " + ex1.Source, MessageBoxButtons.OK, MessageBoxIcon.Error);
            }
            catch (Exception ex2)
            {
                MessageBox.Show(ex2.Message, "Turnos: " + ex2.Source, MessageBoxButtons.OK, MessageBoxIcon.Error);
            }
        }


        private void cargarTablaMesasLibres(DateTime fecha, string hora)
        {
            try
            {
                //netejem la llista per si te dades
                dataGridView_Mesas.ClearSelection();
                dataGridView_Mesas.Rows.Clear();
                //recuperem les taules de la BBDD
                mesas = client.ListarMesasLibres(fecha, hora).ToList();
                for (int i = 0; i < mesas.Count(); i++)
                {//Carreguem la llista de taules                                      
                    dataGridView_Mesas.Rows.Add();
                    dataGridView_Mesas.Rows[i].Cells[0].Value = mesas.ElementAt(i).numero;
                    dataGridView_Mesas.Rows[i].Cells[1].Value = mesas.ElementAt(i).comensales;

                }

                if(this.dataGridView_Mesas.RowCount == 0)
                {
                    MessageBox.Show("No hay mesas libres. Cambie su selección de Día/Turno", "Aforo", MessageBoxButtons.OK, MessageBoxIcon.Information);
                }
            }
            catch (NullReferenceException ex)
            {
                MessageBox.Show(ex.Message, "Mesas: " + ex.Source, MessageBoxButtons.OK, MessageBoxIcon.Error);
            }
            catch (SystemException ex1)
            {
                MessageBox.Show(ex1.Message, "Mesas: " + ex1.Source, MessageBoxButtons.OK, MessageBoxIcon.Error);
            }
            catch (Exception ex2)
            {
                MessageBox.Show(ex2.Message, "Mesas: " + ex2.Source, MessageBoxButtons.OK, MessageBoxIcon.Error);
            }
        }

        

        private void textBox_comensales_Leave(object sender, EventArgs e)
        {
            try
            {
                int.Parse(this.textBox_comensales.Text); //validem que s'ingressa un nº
                if(int.Parse(this.textBox_comensales.Text)<0) //validem que s'ingressa un nº positiu
                {
                    MessageBox.Show("Formato Incorrecto. Ingrese un nº positivo", "Número de Comensales", MessageBoxButtons.OK, MessageBoxIcon.Error);
                    this.textBox_comensales.Clear();
                    this.textBox_comensales.Focus();
                }
            }
            catch
            {
                MessageBox.Show("Formato Incorrecto. Ingrese un nº positivo", "Número de Comensales", MessageBoxButtons.OK, MessageBoxIcon.Error);
                this.textBox_comensales.Clear();
                this.textBox_comensales.Focus();
            }
        }

        private void button_comprobar_Click(object sender, EventArgs e)
        {
            if (this.comboBox_turnos.SelectedItem == null)
            {
                MessageBox.Show("Seleccione un turno", "Nueva Reserva");
            }
            else 
            {
                if ((textBox_nombre.Text.Length > 0) && (textBox_telef.Text.Length > 0) && (textBox_mail.Text.Length > 0)
                    && (textBox_comensales.Text.Length > 0))
                {
                   
                    this.panel2.Visible = true;
                    this.panel1.Enabled = false;
                    this.panel3.Enabled = false;
                    cargarTablaMesasLibres(this.dateTimePicker1.Value.Date, this.comboBox_turnos.SelectedItem.ToString());
                    this.label_reserva.Text = "Datos Reserva:"+ "\n"+" Nombre: "+textBox_nombre.Text
                        + "\n"
                        + "Telefono: "+ textBox_telef.Text
                        + "\n"
                        + "Mail: " + textBox_mail.Text
                        + "\n"
                        + "Comensales: " + textBox_comensales.Text;
                }
                else
                {
                    MessageBox.Show("Faltan Datos", "Nueva Reserva");
                }
            }

            
        }

       
        private void label_validarCapacidad_Click(object sender, EventArgs e)
        {
            this.label_validarCapacidad.Visible = false;
        }

        private void button_volver_Click(object sender, EventArgs e)
        {
            this.panel1.Enabled = true;
            this.panel3.Enabled = true;
            this.panel2.Visible = false;
            this.dataGridView_Mesas.Rows.Clear();
                this.label_reserva.Text = "Datos Reserva: ";
                this.label_mesa.Text = "Mesa: ";
        }

        private void dateTimePicker1_ValueChanged(object sender, EventArgs e)
        {
            //comprovem que la data no es festiva
            bool festivo = client.BuscarFestivo(this.dateTimePicker1.Value.Date);
            if (this.dateTimePicker1.Value.Date < DateTime.Now.Date)
            {
                this.panel1.Visible = false;
                this.label_festivo.Visible = false;
                this.label_anterior.Visible = true;
            }
                //comprovem que la data no és inferior a la data actual
            else if(festivo == true)
                {
                    this.panel1.Visible = false;
                    this.label_festivo.Visible = true;
                    this.label_anterior.Visible = false;
                }

            
            else {
                this.panel1.Visible = true;
                this.label_festivo.Visible = false;
                this.label_anterior.Visible = false;
            }

        }

        private void dataGridView_Mesas_CellClick(object sender, DataGridViewCellEventArgs e)
        {
            this.label_validarCapacidad.Visible = false;
            int personas = int.Parse(this.textBox_comensales.Text);
            DataGridViewRow row = this.dataGridView_Mesas.CurrentRow; //fila seleccionada
            int capacidad = Convert.ToInt32(row.Cells[1].Value); //capacitat de la taula seleccionada
            if (capacidad >= personas)
            {
                this.label_mesa.Text = "Mesa Selecc: " + (row.Cells[0].Value).ToString();
                mesaRes.comensales = Convert.ToInt32(row.Cells[1].Value);
                mesaRes.numero = Convert.ToInt32(row.Cells[0].Value);
            }
            else
            {
                this.label_validarCapacidad.Focus();
                this.label_validarCapacidad.Visible = true;
            }
        }

       
        
    }
}
