﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Windows.Forms;
using Administración.ServiceReference1;

namespace Administración
{
    public partial class UICalendario : Form
    {
        private ServiceReference1.Service1Client client; //variable per accedir al servei WCF       
        private List<Festivo> festivos;
        
        public UICalendario()
        {
            InitializeComponent();
            client = new ServiceReference1.Service1Client();//nova instancia del servei WCF
        }

        private void UICalendario_Load(object sender, EventArgs e)
        {
            this.monthCalendar2.SelectionStart = DateTime.Now;
            cargarListaFestivos();
        }

        private void monthCalendar2_DateSelected(object sender, DateRangeEventArgs e)
        {
            // Muestra la fecha seleccionada en el text box            
            this.textBox_dia.Text = e.End.ToShortDateString();
           
            }

        private void button_marcarFestivo_Click(object sender, EventArgs e)
        {
            try
            {
                if (this.textBox_dia.Text.Length>0) //comprovem que s'ha seleccionat una data
                {
                    DateTime nuevoFestivo = new DateTime();
                    nuevoFestivo = DateTime.Parse(this.textBox_dia.Text);
                    if (client.BuscarFestivo(DateTime.Parse(this.textBox_dia.Text)) == true)
                    {//comprovem que la data és nova
                        MessageBox.Show("Esta Fecha ya está marcada como festivo", "Nuevo Festivo", MessageBoxButtons.OK, MessageBoxIcon.Warning);
                        this.textBox_dia.Clear();
                    } 
                    else { //fem l'alta de festiu
                        client.AltaFestivo(nuevoFestivo);
                        MessageBox.Show("Alta realizada con éxito", "Nuevo Festivo", MessageBoxButtons.OK, MessageBoxIcon.Information);
                        cargarListaFestivos(); //actualitzem la llista
                    }
                    
                }
                else {
                    MessageBox.Show("No se ha seleccionado ninguna fecha", "Nuevo Festivo", MessageBoxButtons.OK, MessageBoxIcon.Warning);
                }

            }
            catch (NullReferenceException ex)
            {
                MessageBox.Show(ex.Message, "Nuevo Festivo: " + ex.Source, MessageBoxButtons.OK, MessageBoxIcon.Error);
            }
            catch (SystemException ex1)
            {
                MessageBox.Show(ex1.Message, "Nuevo Festivo: " + ex1.Source, MessageBoxButtons.OK, MessageBoxIcon.Error);
            }
            catch (Exception ex2)
            {
                MessageBox.Show(ex2.Message, "Nuevo Festivo: " + ex2.Source, MessageBoxButtons.OK, MessageBoxIcon.Error);
            }
        }

        
       

        private void button_desmarcarFestivo_Click(object sender, EventArgs e)
        {
            try
            {
                if (this.textBox_dia.Text.Length>0) //comprovem que s'ha seleccionat una data
                {
                    DateTime oldFestivo = new DateTime();
                    oldFestivo = DateTime.Parse(this.textBox_dia.Text);
                    if (client.BuscarFestivo(DateTime.Parse(this.textBox_dia.Text)) == false)
                    {//comprovem que la data existeix
                        MessageBox.Show("Esta Fecha no está marcada como festivo", "Eliminar Festivo", MessageBoxButtons.OK, MessageBoxIcon.Warning);
                        this.textBox_dia.Clear();
                    }
                    else
                    { //fem l'alta de festiu
                        client.EliminaFestivo(oldFestivo);
                        MessageBox.Show("Baja realizada con éxito", "Eliminar Festivo", MessageBoxButtons.OK, MessageBoxIcon.Information);
                        cargarListaFestivos(); //actualitzem la llista
                    }

                }
                else
                {
                    MessageBox.Show("No se ha seleccionado ninguna fecha", "Eliminar Festivo", MessageBoxButtons.OK, MessageBoxIcon.Warning);
                }

            }
            catch (NullReferenceException ex)
            {
                MessageBox.Show(ex.Message, "Eliminar Festivo: " + ex.Source, MessageBoxButtons.OK, MessageBoxIcon.Error);
            }
            catch (SystemException ex1)
            {
                MessageBox.Show(ex1.Message, "Eliminar Festivo: " + ex1.Source, MessageBoxButtons.OK, MessageBoxIcon.Error);
            }
            catch (Exception ex2)
            {
                MessageBox.Show(ex2.Message, "Eliminar Festivo: " + ex2.Source, MessageBoxButtons.OK, MessageBoxIcon.Error);
            }

        }


        private void cargarListaFestivos()
        {
            try
            {
                int year = DateTime.Now.Year;
                //netejem la llista per si te dades
                listBox_festivos.Items.Clear();
                //recuperem els festius de la BBDD
                festivos = client.ListarFestivos().ToList();
                for (int i = 0; i < festivos.Count(); i++)
                {//Carreguem la llista de festius si són de l'any actual o posterior
                    if (festivos.ElementAt(i).Fecha.Year >= year)
                    {
                        listBox_festivos.Items.Add(festivos.ElementAt(i).Fecha);
                    }
                }
            }
            catch (NullReferenceException ex)
            {
                MessageBox.Show(ex.Message, "Festivos: " + ex.Source, MessageBoxButtons.OK, MessageBoxIcon.Error);
            }
            catch (SystemException ex1)
            {
                MessageBox.Show(ex1.Message, "Festivos: " + ex1.Source, MessageBoxButtons.OK, MessageBoxIcon.Error);
            }
            catch (Exception ex2)
            {
                MessageBox.Show(ex2.Message, "Festivos: " + ex2.Source, MessageBoxButtons.OK, MessageBoxIcon.Error);
            }
        }

       
        
        private void button_cerrar_Click(object sender, EventArgs e)
        {
            this.Dispose();
        }

    }
}
