﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Windows.Forms;
using Administración.ServiceReference1;

namespace Administración
{
    public partial class UICartas : Form
    {
        private ServiceReference1.Service1Client client; //variable per accedir al servei WCF
        private List<Carta> cartas;

        public UICartas()
        {
            InitializeComponent();
            client = new ServiceReference1.Service1Client();//nova instancia del servei WCF
        }

        private void button_Cancelar_Click(object sender, EventArgs e)
        {
            this.Dispose();
        }

        private void button_EliminarCarta_Click(object sender, EventArgs e)
        {
            try
            {
                DialogResult result = MessageBox.Show("Confirme la Eliminación", "Eliminar Carta", MessageBoxButtons.OKCancel, MessageBoxIcon.Question);

                if (result == DialogResult.OK)
                {
                    DataGridViewRow row = this.dataGridView_Cartas.CurrentRow; //fila seleccionada
                    int idCarta = Convert.ToInt32(row.Cells[0].Value);
                    client.EliminarCarta(idCarta);
                    this.dataGridView_Cartas.Rows.Clear();
                    cargarCartas();
                }
                else if (result == DialogResult.Cancel)
                {
                }

            }
            catch (NullReferenceException ex)
            {
                MessageBox.Show("No se ha seleccionado ninguna carta", "Cartas: " + ex.Source, MessageBoxButtons.OK, MessageBoxIcon.Information);
            }
            catch (SystemException ex1)
            {
                MessageBox.Show(ex1.Message, "Cartas: " + ex1.Source, MessageBoxButtons.OK, MessageBoxIcon.Error);
            }
            catch (Exception ex2)
            {
                MessageBox.Show(ex2.Message, "Cartas: " + ex2.Source, MessageBoxButtons.OK, MessageBoxIcon.Error);
            }

        }

        private void button_NuevaCarta_Click(object sender, EventArgs e)
        {
            this.dataGridView_Cartas.Rows.Clear();
            UIAltaCarta f = new UIAltaCarta(null);
            f.ShowDialog();
        }

        private void button_EditarCarta_Click(object sender, EventArgs e)
        {
            try{
            DataGridViewRow row = this.dataGridView_Cartas.CurrentRow; //fila seleccionada            
            Carta cartaModif = new Carta();
            cartaModif.Id = Convert.ToInt32(row.Cells[0].Value);
            cartaModif.Nombre = Convert.ToString(row.Cells[1].Value);
            cartaModif.Fecha = Convert.ToDateTime(row.Cells[2].Value);

            this.dataGridView_Cartas.Rows.Clear();
            UIAltaCarta f = new UIAltaCarta(cartaModif);
            f.ShowDialog();

            }
            catch (NullReferenceException ex)
            {
                MessageBox.Show("No se ha seleccionado ninguna carta", "Cartas: " + ex.Source, MessageBoxButtons.OK, MessageBoxIcon.Information);
            }
            catch (SystemException ex1)
            {
                MessageBox.Show(ex1.Message, "Cartas: " + ex1.Source, MessageBoxButtons.OK, MessageBoxIcon.Error);
            }
            catch (Exception ex2)
            {
                MessageBox.Show(ex2.Message, "Cartas: " + ex2.Source, MessageBoxButtons.OK, MessageBoxIcon.Error);
            }
        }

        //mètode per carregar les cartes de la BBDD a la taula
        protected void cargarCartas()
        {
            try
            {
                //netejem la llista per si te dades
                dataGridView_Cartas.ClearSelection();
                //recuperem els plats de la BBDD
                cartas = client.ListarCartas().ToList();
                for (int i = 0; i < cartas.Count(); i++)
                {//Carreguem la llista de plats   
                    dataGridView_Cartas.Rows.Add();
                    dataGridView_Cartas.Rows[i].Cells[0].Value = cartas.ElementAt(i).Id;//columna oculta
                    dataGridView_Cartas.Rows[i].Cells[1].Value = cartas.ElementAt(i).Nombre;
                    dataGridView_Cartas.Rows[i].Cells[2].Value = cartas.ElementAt(i).Fecha.ToShortDateString();
                    dataGridView_Cartas.Rows[i].Cells[3].Value = cartas.ElementAt(i).Publicar;
                }
            }
            catch (NullReferenceException ex)
            {
                MessageBox.Show(ex.Message, "Cartas: " + ex.Source, MessageBoxButtons.OK, MessageBoxIcon.Error);
            }
            catch (SystemException ex1)
            {
                MessageBox.Show(ex1.Message, "Cartas: " + ex1.Source, MessageBoxButtons.OK, MessageBoxIcon.Error);
            }
            catch (Exception ex2)
            {
                MessageBox.Show(ex2.Message, "Cartas: " + ex2.Source, MessageBoxButtons.OK, MessageBoxIcon.Error);
            }
        }

        protected void UICartas_Load(object sender, EventArgs e)
        {
            this.dataGridView_Cartas.Rows.Clear();
            cargarCartas();
        }

        private void dataGridView_Cartas_SelectionChanged(object sender, EventArgs e)
        {
            this.button_EditarCarta.Enabled = true;
            this.button_EliminarCarta.Enabled = true;
            this.button_ActualizCarta.Enabled = true;
        }

        private void button_ActualizCarta_Click(object sender, EventArgs e)
        {
            try
            {
                DataGridViewRow row = this.dataGridView_Cartas.CurrentRow; //fila seleccionada
                int idCarta = Convert.ToInt32(row.Cells[0].Value);
                client.DespublicarCarta();
                client.PublicarCarta(idCarta);
                this.dataGridView_Cartas.Rows.Clear();
                cargarCartas();
                
            }
            catch (NullReferenceException ex)
            {
                MessageBox.Show(ex.Message, "Cartas: " + ex.Source, MessageBoxButtons.OK, MessageBoxIcon.Error);
            }
            catch (SystemException ex1)
            {
                MessageBox.Show(ex1.Message, "Cartas: " + ex1.Source, MessageBoxButtons.OK, MessageBoxIcon.Error);
            }
            catch (Exception ex2)
            {
                MessageBox.Show(ex2.Message, "Cartas: " + ex2.Source, MessageBoxButtons.OK, MessageBoxIcon.Error);
            }
        }

        private void button_actualizar_Click(object sender, EventArgs e)
        {
            this.dataGridView_Cartas.Rows.Clear();
            cargarCartas();
        }
    }
}
