﻿using System;
using System.Resources;
using System.Collections;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Net.Mail;
using System.Text;
using System.Windows.Forms;
using Administración.ServiceReference1;

namespace Administración
{
    public partial class UINoticias : Form
    {
        private ServiceReference1.Service1Client client; //variable per accedir al servei WCF
        private List<Oferta> noticias;
        private List<String> listaDistribucion;

        public UINoticias()
        {
            InitializeComponent();
            client = new ServiceReference1.Service1Client();//nova instancia del servei WCF
        }

        private void button_cerrar_Click(object sender, EventArgs e)
        {
            this.Dispose();
        }

        private void button_añadir_Click(object sender, EventArgs e)
        {
            this.dataGridView_Ofertas.Rows.Clear();//netegem la llista
            Oferta oferta = null;
            UIAltaNoticia f = new UIAltaNoticia(oferta);
                f.ShowDialog();
        }

        private void button_editar_Click(object sender, EventArgs e)
        {
            int numFilas = this.dataGridView_Ofertas.RowCount;
            if (numFilas > 0)//comprovem que s'ha sel.leccionat alguna noticia
            {
                //recuperem la noticia seleccionada
                DataGridViewRow row = this.dataGridView_Ofertas.CurrentRow;

                Oferta ofertaModif = new Oferta();
                ofertaModif.Id = Convert.ToInt32(row.Cells[0].Value);
                ofertaModif.Fecha = Convert.ToDateTime(row.Cells[1].Value);
                ofertaModif.Nombre = Convert.ToString(row.Cells[2].Value);
                ofertaModif.Contenido = Convert.ToString(Convert.ToString(row.Cells[3].Value));

                this.dataGridView_Ofertas.Rows.Clear();//netegem la llista
                UIAltaNoticia f = new UIAltaNoticia(ofertaModif);
                f.ShowDialog();
            }
            else
            {
                MessageBox.Show("No se ha seleccionado ninguna noticia", "Modificar Noticia", MessageBoxButtons.OK, MessageBoxIcon.Information);
            }
        }

        private void dataGridView_Ofertas_SelectionChanged(object sender, EventArgs e)
        {
            this.button_editar.Enabled = true;
        }

        //mètode per carregar les notícies de la BBDD a la taula
        private void cargarNoticias()
        {
            try
            {
                //netejem la llista per si te dades
                dataGridView_Ofertas.ClearSelection();
                //recuperem les noticies de la BBDD
                this.noticias = client.ListarOfertas().ToList();
                for (int i = 0; i < this.noticias.Count(); i++)
                {//Carreguem la llista de noticies 
                    dataGridView_Ofertas.Rows.Add();
                    dataGridView_Ofertas.Rows[i].Cells[0].Value = noticias.ElementAt(i).Id;//columna oculta
                    dataGridView_Ofertas.Rows[i].Cells[1].Value = noticias.ElementAt(i).Fecha.ToShortDateString();
                    dataGridView_Ofertas.Rows[i].Cells[2].Value = noticias.ElementAt(i).Nombre;
                    dataGridView_Ofertas.Rows[i].Cells[3].Value = noticias.ElementAt(i).Contenido;
                }
            }
            catch (NullReferenceException ex)
            {
                MessageBox.Show(ex.Message, "Noticias: " + ex.Source, MessageBoxButtons.OK, MessageBoxIcon.Error);
            }
            catch (SystemException ex1)
            {
                MessageBox.Show(ex1.Message, "Noticias: " + ex1.Source, MessageBoxButtons.OK, MessageBoxIcon.Error);
            }
            catch (Exception ex2)
            {
                MessageBox.Show(ex2.Message, "Noticias: " + ex2.Source, MessageBoxButtons.OK, MessageBoxIcon.Error);
            }
        }

        private void UINoticias_Load(object sender, EventArgs e)
        {
            cargarNoticias();
        }

        private void button_actualizar_Click(object sender, EventArgs e)
        {
            this.dataGridView_Ofertas.Rows.Clear();
            cargarNoticias();
        }

        private void button_enviar_Click(object sender, EventArgs e)
        {
            //recuperem la llista de subscripció
            this.listaDistribucion = client.ListaDistribucionNoticias().ToList();

            //compovem que hi ha subscriptors
            if (listaDistribucion.Count > 0)
            {
                int numFilas = this.dataGridView_Ofertas.RowCount;
                if (numFilas > 0)//comprovem que s'ha sel.leccionat alguna noticia
                {
                    //recuperem la noticia que s'ha d'enviar
                    DataGridViewRow row = this.dataGridView_Ofertas.CurrentRow;
                    string fechaNoticia = Convert.ToString(row.Cells[1].Value.ToString());
                    string tituloNoticia = Convert.ToString(row.Cells[2].Value);
                    string contenidoNoticia = Convert.ToString(Convert.ToString(row.Cells[3].Value));

                    //recuperem les dades del nostre servidor
                    string smtpEmail = Administración.Properties.Resources.eMail;
                    string smtpPw = Administración.Properties.Resources.password;
                    string smtpServer = Administración.Properties.Resources.ServidorSmtp;

                    //creem el missatge de mail
                    MailMessage correo = new MailMessage();
                    correo.From = new MailAddress(smtpEmail);


                    for (int i = 0; i < this.listaDistribucion.Count(); i++)
                    {//Carreguem la llista de mails   
                        correo.To.Add(listaDistribucion.ElementAt(i).ToString());
                    }

                    correo.Subject = "Noticias UOC Restaurant";

                    correo.Body = fechaNoticia + " - " + tituloNoticia + "\n" + contenidoNoticia;
                    correo.IsBodyHtml = false;
                    correo.Priority = MailPriority.Normal;
                    SmtpClient smtp = new SmtpClient(smtpServer);
                    smtp.EnableSsl = true;
                    smtp.UseDefaultCredentials = false;
                    smtp.Port = 25;
                    smtp.Credentials = new System.Net.NetworkCredential(smtpEmail, smtpPw);

                    try
                    {//enviem el missatge
                        smtp.Send(correo);
                        MessageBox.Show("Envio realizado con éxito", "Enviar Noticias", MessageBoxButtons.OK, MessageBoxIcon.Information);
                    }
                    catch (Exception ex)
                    {
                        MessageBox.Show(ex.Message, "Enviar Noticias: " + ex.Source, MessageBoxButtons.OK, MessageBoxIcon.Error);
                    }

                }
                else
                {
                    MessageBox.Show("No se ha seleccionado ninguna noticia", "Enviar Noticias", MessageBoxButtons.OK, MessageBoxIcon.Information);
                }
            }
            else
            {
                MessageBox.Show("Sin suscriptores", "Envio Noticias: ", MessageBoxButtons.OK, MessageBoxIcon.Information);
            }


            
        }

        

    }
}
