﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Windows.Forms;
using Administración.ServiceReference1;

namespace Administración
{
    public partial class UIReservas : Form
    {
        private ServiceReference1.Service1Client client; //variable per accedir al servei WCF
        private List<Reserva> reservas;
        private Reserva reserva;

        public UIReservas()
        {
            InitializeComponent();
            client = new ServiceReference1.Service1Client();//nova instancia del servei WCF
        }

        private void UIReservas_Load(object sender, EventArgs e)
        {
            this.dateTimePicker1.Value = DateTime.Now;
        }


        private void button_eliminar_Click(object sender, EventArgs e)
        {
            int numFilas = this.dataGridView_reservas.RowCount;
            if (numFilas > 0)//comprovem que s'ha sel.leccionat alguna reserva
            {
                try
                {
                    //recuperem la reserva selecionada
                    DataGridViewRow row = this.dataGridView_reservas.CurrentRow;
                    int reservaEliminar = Convert.ToInt32(row.Cells[0].Value);
                    DateTime fechaEliminar = (Convert.ToDateTime(row.Cells[1].Value)).Date;
                    String turnoEliminar = Convert.ToString(row.Cells[2].Value);
                    int mesaEliminar = Convert.ToInt32(row.Cells[3].Value);
                    
                        DialogResult result = MessageBox.Show("Confirme la Eliminación", "Eliminar Reserva", MessageBoxButtons.OKCancel, MessageBoxIcon.Question);

                        if (result == DialogResult.OK)
                        {
                            client.EliminarReserva(reservaEliminar, fechaEliminar, mesaEliminar, turnoEliminar);
                            //netejem la taula de files
                            dataGridView_reservas.Rows.Clear();
                            //actualitzem les dades a la taula
                        }
                }
                catch (NullReferenceException ex)
                {
                    MessageBox.Show(ex.Message, "Reservas: " + ex.Source, MessageBoxButtons.OK, MessageBoxIcon.Error);
                }
                catch (SystemException ex1)
                {
                    MessageBox.Show(ex1.Message, "Reservas: " + ex1.Source, MessageBoxButtons.OK, MessageBoxIcon.Error);
                }
                catch (Exception ex2)
                {
                    MessageBox.Show(ex2.Message, "Reservas: " + ex2.Source, MessageBoxButtons.OK, MessageBoxIcon.Error);
                }
            }
            else
            {
                MessageBox.Show("No se ha seleccionado ninguna reserva", "Eliminar Reserva", MessageBoxButtons.OK, MessageBoxIcon.Information);
            }

        }
      

        private void button_nueva_Click(object sender, EventArgs e)
        {
            
            UIAltaReserva f = new UIAltaReserva(null);
            //netejem la taula de files
            dataGridView_reservas.Rows.Clear();
            f.ShowDialog();
        }
       

       
        private void button_filtrar_Click(object sender, EventArgs e)
        {
            try
            {int.Parse(this.textBox_numReserva.Text);//verifiquem que s'ha entrat un número
                
                //netejem la taula de files
                dataGridView_reservas.Rows.Clear();
                //carreguem les dades a la taula
                 this.cargarReservaNumero(int.Parse(this.textBox_numReserva.Text));

                 this.textBox_ocupacion.Clear();
            }
            catch (NullReferenceException ex)
            {
                MessageBox.Show(ex.Message, "Ver Reservas: " + ex.Source, MessageBoxButtons.OK, MessageBoxIcon.Error);
            }
            catch (SystemException ex1)
            {
                MessageBox.Show("No se ha informado ningún nº de reserva", "Ver Reservas: " , MessageBoxButtons.OK, MessageBoxIcon.Error);
            }
            catch (Exception ex2)
            {
                MessageBox.Show(ex2.Message, "Ver Reservas: " + ex2.Source, MessageBoxButtons.OK, MessageBoxIcon.Error);
            }
        }

        //al canviar al data de selcció es genera la taula amb les reserves
        private void dateTimePicker1_ValueChanged(object sender, EventArgs e)
        {
            //netejem la taula de files
            dataGridView_reservas.Rows.Clear();
            cargarReservasFecha(dateTimePicker1.Value.Date);
           
        }

        private void cargarReservasFecha(DateTime pFecha)
        {
            try
            {
                //netejem la llista per si te dades
                dataGridView_reservas.ClearSelection();
                //recuperem les reserves de la BBDD
                reservas = client.ListarReservasFecha(pFecha).ToList();
                for (int i = 0; i < reservas.Count(); i++)
                {//Carreguem la llista de reserves   
                    dataGridView_reservas.Rows.Add();
                    dataGridView_reservas.Rows[i].Cells[0].Value = reservas.ElementAt(i).id;
                    dataGridView_reservas.Rows[i].Cells[1].Value = reservas.ElementAt(i).Fecha.ToShortDateString();
                    dataGridView_reservas.Rows[i].Cells[2].Value = reservas.ElementAt(i).Turno;
                    dataGridView_reservas.Rows[i].Cells[3].Value = reservas.ElementAt(i).Mesa;
                    dataGridView_reservas.Rows[i].Cells[4].Value = reservas.ElementAt(i).Nombre;
                    dataGridView_reservas.Rows[i].Cells[5].Value = reservas.ElementAt(i).Comensales;
                    dataGridView_reservas.Rows[i].Cells[6].Value = reservas.ElementAt(i).Telefono;
                    dataGridView_reservas.Rows[i].Cells[7].Value = reservas.ElementAt(i).Mail;
                }

                //calculem l'ocupació
                this.textBox_ocupacion.Text = plazasOcupadas(reservas);
            }
            catch (NullReferenceException ex)
            {
                MessageBox.Show(ex.Message, "Reservas: " + ex.Source, MessageBoxButtons.OK, MessageBoxIcon.Error);
            }
            catch (SystemException ex1)
            {
                MessageBox.Show(ex1.Message, "Reservas: " + ex1.Source, MessageBoxButtons.OK, MessageBoxIcon.Error);
            }
            catch (Exception ex2)
            {
                MessageBox.Show(ex2.Message, "Reservas: " + ex2.Source, MessageBoxButtons.OK, MessageBoxIcon.Error);
            }
          }


        private void cargarReservaNumero(int idReserva)
        {
            try
            {
                //netejem la llista per si te dades
                dataGridView_reservas.ClearSelection();
                //recuperem les reserves de la BBDD
                reservas = client.ListarReservasNumero(idReserva).ToList();
                for (int i = 0; i < reservas.Count(); i++)
                {//Carreguem la llista de reserves  
                    dataGridView_reservas.Rows.Add();
                    dataGridView_reservas.Rows[i].Cells[0].Value = reservas.ElementAt(i).id;
                    dataGridView_reservas.Rows[i].Cells[1].Value = reservas.ElementAt(i).Fecha.ToShortDateString();
                    dataGridView_reservas.Rows[i].Cells[2].Value = reservas.ElementAt(i).Turno;
                    dataGridView_reservas.Rows[i].Cells[3].Value = reservas.ElementAt(i).Mesa;
                    dataGridView_reservas.Rows[i].Cells[4].Value = reservas.ElementAt(i).Nombre;
                    dataGridView_reservas.Rows[i].Cells[5].Value = reservas.ElementAt(i).Comensales;
                    dataGridView_reservas.Rows[i].Cells[6].Value = reservas.ElementAt(i).Telefono;
                    dataGridView_reservas.Rows[i].Cells[7].Value = reservas.ElementAt(i).Mail;
                }
            }
            catch (NullReferenceException ex)
            {
                MessageBox.Show(ex.Message, "Reservas: " + ex.Source, MessageBoxButtons.OK, MessageBoxIcon.Error);
            }
            catch (SystemException ex1)
            {
                MessageBox.Show(ex1.Message, "Reservas: " + ex1.Source, MessageBoxButtons.OK, MessageBoxIcon.Error);
            }
            catch (Exception ex2)
            {
                MessageBox.Show(ex2.Message, "Reservas: " + ex2.Source, MessageBoxButtons.OK, MessageBoxIcon.Error);
            }
        }

        private void button_cerrar_Click(object sender, EventArgs e)
        {
            this.Dispose();
        }

        //mmètode per calcular l'ocupació
        private string plazasOcupadas(List<Reserva> reservas)
        {
            int mesas = 0;
            int personas = 0;
            for (int i = 0; i < reservas.Count(); i++)
            {//calculem l'ocupació 
                mesas = mesas + 1;
                personas = personas + reservas.ElementAt(i).Comensales;
            }
            return ("Mesas Ocupadas: " + mesas + " - Comensales: " + personas);
        }


        private void button_modificar_Click(object sender, EventArgs e)
        {
            int numFilas = this.dataGridView_reservas.RowCount;
            if (numFilas > 0)//comprovem que s'ha sel.leccionat alguna reserva
            {
                try
                {
                    //recuperem la reserva selecionada
                    DataGridViewRow row = this.dataGridView_reservas.CurrentRow;
                    Reserva oldReserva = new Reserva();
                    oldReserva.id = Convert.ToInt32(row.Cells[0].Value);
                    oldReserva.Fecha = (Convert.ToDateTime(row.Cells[1].Value)).Date;
                    oldReserva.Turno = Convert.ToString(row.Cells[2].Value);
                    oldReserva.Mesa = Convert.ToInt32(row.Cells[3].Value);
                    oldReserva.Nombre = Convert.ToString(row.Cells[4].Value);
                    oldReserva.Comensales = Convert.ToInt32(row.Cells[5].Value);
                    oldReserva.Telefono = Convert.ToString(row.Cells[6].Value);
                    oldReserva.Mail = Convert.ToString(row.Cells[7].Value);

                    
                    UIAltaReserva f = new UIAltaReserva(oldReserva);
                    //netejem la taula de files
                    dataGridView_reservas.Rows.Clear();
                    f.ShowDialog();
                }
                catch (NullReferenceException ex)
                {
                    MessageBox.Show(ex.Message, "Reservas: " + ex.Source, MessageBoxButtons.OK, MessageBoxIcon.Error);
                }
                catch (SystemException ex1)
                {
                    MessageBox.Show(ex1.Message, "Reservas: " + ex1.Source, MessageBoxButtons.OK, MessageBoxIcon.Error);
                }
                catch (Exception ex2)
                {
                    MessageBox.Show(ex2.Message, "Reservas: " + ex2.Source, MessageBoxButtons.OK, MessageBoxIcon.Error);
                }
            }
            else
            {
                MessageBox.Show("No se ha seleccionado ninguna reserva", "Eliminar Reserva", MessageBoxButtons.OK, MessageBoxIcon.Information);
            }
        }

      

       
    }
}
