﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Windows.Forms;
using Administración.ServiceReference1;

namespace Administración
{
    public partial class UISecciones : Form
    {
        private ServiceReference1.Service1Client client; //variable per accedir al servei WCF   
        private List<Seccion> secciones;
        private List<Tipo> tipos;

        public UISecciones()
        {
            InitializeComponent();
            client = new ServiceReference1.Service1Client();//nova instancia del servei WCF
        }

        private void button_Cerrar_Click(object sender, EventArgs e)
        {
            this.Dispose();
        }

        protected void UISecciones_Load(object sender, EventArgs e)
        {
            //al carregar la finestra es carrega la llista de seccions i de tipus
            cargarListaSecciones();
            cargarListaTipos();
        }

        
        private void button_nuevaSeccion_Click(object sender, EventArgs e)
        {
            try
            {
                if (this.textBox_NuevaSeccion.Text.Length == 0)
                {
                    //no s'ha escrit cap nom per la nova secció
                    MessageBox.Show("Debe escribir un nombre para la sección", "Nueva Sección", MessageBoxButtons.OK, MessageBoxIcon.Warning);
                }
                else
                {
                //comprovem si el nom de la nova secció és nou
                if(client.BuscarSeccion(textBox_NuevaSeccion.Text)== true)
                {
                    //la secció ja existeix i no es pot fer l'alta
                    MessageBox.Show("Esta sección ya existe. No se puede efectuar el alta", "Nueva Sección", MessageBoxButtons.OK, MessageBoxIcon.Warning);
                    textBox_NuevaSeccion.Clear();
                }
                else
                {
                    //si la secció no existeix fem l'alta
                client.AltaSeccion(textBox_NuevaSeccion.Text);
                textBox_NuevaSeccion.Clear();
                    //actualitzem la llista de seccions
                cargarListaSecciones();
                }
                }
            }
            catch (NullReferenceException ex)
            {
                MessageBox.Show(ex.Message, "Secciones: " + ex.Source, MessageBoxButtons.OK, MessageBoxIcon.Error);
            }
            catch (SystemException ex1)
            {
                MessageBox.Show(ex1.Message, "Secciones: " + ex1.Source, MessageBoxButtons.OK, MessageBoxIcon.Error);
            }
            catch (Exception ex2)
            {
                MessageBox.Show(ex2.Message, "Secciones: " + ex2.Source, MessageBoxButtons.OK, MessageBoxIcon.Error);
            }
        }
        

        private void cargarListaSecciones()
        {
            try
            {
                //netejem la llista per si te dades
                listBox_secciones.Items.Clear();
                //recuperem les seccions de la BBDD
                secciones = client.ListarSecciones().ToList();
                for (int i = 0; i < secciones.Count(); i++)
                {//Carreguem la llista de seccions
                    listBox_secciones.Items.Add(secciones.ElementAt(i).nombre);
                }
            }
            catch (NullReferenceException ex)
            {
                MessageBox.Show(ex.Message, "Secciones: " + ex.Source, MessageBoxButtons.OK, MessageBoxIcon.Error);
            }
            catch (SystemException ex1)
            {
                MessageBox.Show(ex1.Message, "Secciones: " + ex1.Source, MessageBoxButtons.OK, MessageBoxIcon.Error);
            }
            catch (Exception ex2)
            {
                MessageBox.Show(ex2.Message, "Secciones: " + ex2.Source, MessageBoxButtons.OK, MessageBoxIcon.Error);
            }
        }

        private void cargarListaTipos()
        {
            try
            {
                //netejem la llista per si te dades
                listBox_tipos.Items.Clear();
                //recuperem els tipus de la BBDD
                tipos = client.ListarTipos().ToList();
                for (int i = 0; i < tipos.Count(); i++)
                {//Carreguem la llista de tipus
                    listBox_tipos.Items.Add(tipos.ElementAt(i).Tipo1);
                }
            }
            catch (NullReferenceException ex)
            {
                MessageBox.Show(ex.Message, "Secciones: " + ex.Source, MessageBoxButtons.OK, MessageBoxIcon.Error);
            }
            catch (SystemException ex1)
            {
                MessageBox.Show(ex1.Message, "Secciones: " + ex1.Source, MessageBoxButtons.OK, MessageBoxIcon.Error);
            }
            catch (Exception ex2)
            {
                MessageBox.Show(ex2.Message, "Secciones: " + ex2.Source, MessageBoxButtons.OK, MessageBoxIcon.Error);
            }
        }

       
        private void button_añadirTipo_Click(object sender, EventArgs e)
        {
            try
            {
                if (this.textBox_newTipo.Text.Length == 0)
                {
                    //no s'a escrit cap nom pel nou tipus
                    MessageBox.Show("Debe escribir un nombre para el tipo de plato", "Nuevo Tipo", MessageBoxButtons.OK, MessageBoxIcon.Warning);
                }
                else
                {
                //comprovem si el nom del nou tipus és nou
                if (client.BuscarTipo(textBox_newTipo.Text) == true)
                {
                    //el tipus ja existeix i no es pot fer l'alta
                    MessageBox.Show("Este tipo ya existe. No se puede efectuar el alta", "Nuevo Tipo", MessageBoxButtons.OK, MessageBoxIcon.Warning);
                    textBox_newTipo.Clear();
                }
                else
                {
                    //si el tipus és nou fem l'alta del nou tipus
                    client.AltaTipo(textBox_newTipo.Text);
                    textBox_newTipo.Clear();
                    //actualitzem la llista de tipus
                    cargarListaTipos();
                }
                }
            }
            catch (NullReferenceException ex)
            {
                MessageBox.Show(ex.Message, "Secciones: " + ex.Source, MessageBoxButtons.OK, MessageBoxIcon.Error);
            }
            catch (SystemException ex1)
            {
                MessageBox.Show(ex1.Message, "Secciones: " + ex1.Source, MessageBoxButtons.OK, MessageBoxIcon.Error);
            }
            catch (Exception ex2)
            {
                MessageBox.Show(ex2.Message, "Secciones: " + ex2.Source, MessageBoxButtons.OK, MessageBoxIcon.Error);
            }
        }

        

        private void listBox_secciones_SelectedIndexChanged(object sender, EventArgs e)
        {
            //al seleccionar un element s'habiliten els botons eliminar/modificar
            button_modifSeccion.Enabled = true;
            button_eliminarSeccion.Enabled = true;
        }

        private void button_modifSeccion_Click(object sender, EventArgs e)
        {
            if (listBox_secciones.Text.Length == 0)//comprovem que s'ha seleccionat algun registre de la llista
            {
                MessageBox.Show("No hay ningún registro seleccionado", "Modificar Seccion", MessageBoxButtons.OK, MessageBoxIcon.Warning);
            }
            else
            {
                this.panel_modifSecc.Visible = true;
                this.panel_nuevaSecc.Enabled = false;
                this.panel_secciones.Enabled = false;
            }
        }

        private void button_eliminarSeccion_Click(object sender, EventArgs e)
        {

            try
            {
                if (listBox_secciones.Text.Length == 0)//comprovem que s'ha seleccionat algun registre de la llista
                {
                    MessageBox.Show("No hay ningún registro seleccionado", "Eliminar Seccion", MessageBoxButtons.OK, MessageBoxIcon.Warning);
                }
                else
                {
                    //recuperem la secció que es vol eliminar
                    int i = listBox_secciones.SelectedIndex;
                    int seccionEliminar = secciones.ElementAt(i).Id;
                    //comprobem que la secció no està assignada a cap plat
                    if (client.BuscarSeccionEnPlato(seccionEliminar) == true)
                    {
                        MessageBox.Show("La sección no se puede eliminar. Está asignada a Platos", "Eliminar Sección", MessageBoxButtons.OK, MessageBoxIcon.Information);
                    }
                    else
                    {
                        DialogResult result = MessageBox.Show("Confirme la Eliminación", "Eliminar Sección", MessageBoxButtons.OKCancel, MessageBoxIcon.Question);

                        if (result == DialogResult.OK)
                        {
                            client.EliminarSeccion(seccionEliminar);
                            cargarListaSecciones();
                        }
                    }
                }
               
            }
            catch (NullReferenceException ex)
            {
                MessageBox.Show(ex.Message, "Secciones: " + ex.Source, MessageBoxButtons.OK, MessageBoxIcon.Error);
            }
            catch (SystemException ex1)
            {
                MessageBox.Show(ex1.Message, "Secciones: " + ex1.Source, MessageBoxButtons.OK, MessageBoxIcon.Error);
            }
            catch (Exception ex2)
            {
                MessageBox.Show(ex2.Message, "Secciones: " + ex2.Source, MessageBoxButtons.OK, MessageBoxIcon.Error);
            }
        }
       
        private void listBox_tipos_SelectedIndexChanged(object sender, EventArgs e)
        {
            //al seleccionar un element s'habiliten els botons eliminar/modificar
            button_modifTipo.Enabled = true;
            button_eliminarTipo.Enabled = true;
        }

        private void button_modifTipo_Click(object sender, EventArgs e)
        {
            if (listBox_tipos.Text.Length == 0)//comprovem que s'ha seleccionat algun registre de la llista
            {
                MessageBox.Show("No hay ningún registro seleccionado", "Modificar Tipo", MessageBoxButtons.OK, MessageBoxIcon.Warning);
            }
            else
            {
                this.panel_modifTipo.Visible = true;
                this.panel_nuevoTipo.Enabled = false;
                this.panel_tipos.Enabled = false;
            }
        }

        

        private void button_eliminarTipo_Click(object sender, EventArgs e)
        {
            try
            {
                if (listBox_tipos.Text.Length == 0)//comprovem que s'ha seleccionat algun registre de la llista
                {
                    MessageBox.Show("No hay ningún registro seleccionado", "Eliminar Tipo", MessageBoxButtons.OK, MessageBoxIcon.Warning);
                }
                else
                {
                //recuperem el tipus que es vol eliminar
                int i = listBox_tipos.SelectedIndex;
                int tipoEliminar = tipos.ElementAt(i).Id;
                //comprobem que la secció no està assignada a cap plat
                if (client.BuscarTipoEnMenu(tipoEliminar) == true)
                {
                    MessageBox.Show("El tipo no se puede eliminar. Está asignado a menú", "Eliminar Tipo", MessageBoxButtons.OK, MessageBoxIcon.Information);
                }
                else
                {
                    DialogResult result = MessageBox.Show("Confirme la Eliminación", "Eliminar Tipo", MessageBoxButtons.OKCancel, MessageBoxIcon.Question);

                    if (result == DialogResult.OK)
                    {
                        client.EliminarTipo(tipoEliminar);
                        cargarListaTipos();
                    }
                }
                }

            }
            catch (NullReferenceException ex)
            {
                MessageBox.Show(ex.Message, "Tipos: " + ex.Source, MessageBoxButtons.OK, MessageBoxIcon.Error);
            }
            catch (SystemException ex1)
            {
                MessageBox.Show(ex1.Message, "Tipos: " + ex1.Source, MessageBoxButtons.OK, MessageBoxIcon.Error);
            }
            catch (Exception ex2)
            {
                MessageBox.Show(ex2.Message, "Tipos: " + ex2.Source, MessageBoxButtons.OK, MessageBoxIcon.Error);
            }
        }

        private void tabControl1_Enter(object sender, EventArgs e)
        {
            cargarListaSecciones();
        }

        private void button_modificar_Click(object sender, EventArgs e)
        {
            try
            {
                if (textBox_modificarSecc.Text.Length == 0)
                {
                    MessageBox.Show("Se debe escribir un nombre para la modificación", "Modificar Sección", MessageBoxButtons.OK, MessageBoxIcon.Warning);
                }
                else
                {
                    //comprovem si el nom de la nova secció és nou
                    if (client.BuscarSeccion(textBox_modificarSecc.Text) == true)
                    {
                        //la secció ja existeix i no es pot fer l'alta
                        MessageBox.Show("Esta sección ya existe. No se puede efectuar la modificación", "Modificar Sección", MessageBoxButtons.OK, MessageBoxIcon.Warning);
                        textBox_modificarSecc.Clear();
                    }
                    else
                    {
                        //recuperem la secció que es vol modificar
                        int i = listBox_secciones.SelectedIndex;
                        Seccion seccionModif = secciones.ElementAt(i);

                        client.ModificarSeccion(seccionModif.Id, textBox_modificarSecc.Text);
                        cargarListaSecciones(); //actualitzem la llista de seccions
                        this.panel_modifSecc.Visible = false;
                        this.panel_nuevaSecc.Enabled = true;
                        this.panel_secciones.Enabled = true;
                        
                    }
                }
            }
            catch (NullReferenceException ex)
            {
                MessageBox.Show(ex.Message, "Secciones: " + ex.Source, MessageBoxButtons.OK, MessageBoxIcon.Error);
            }
            catch (SystemException ex1)
            {
                MessageBox.Show(ex1.Message, "Secciones: " + ex1.Source, MessageBoxButtons.OK, MessageBoxIcon.Error);
            }
            catch (Exception ex2)
            {
                MessageBox.Show(ex2.Message, "Secciones: " + ex2.Source, MessageBoxButtons.OK, MessageBoxIcon.Error);
            }
        }

        private void button_cancelar_Click(object sender, EventArgs e)
        {
            this.panel_modifSecc.Visible = false;
            this.panel_nuevaSecc.Enabled = true;
            this.panel_secciones.Enabled = true;
        }

        private void button_cancelarModifTipo_Click(object sender, EventArgs e)
        {
            this.panel_modifTipo.Visible = false;
            this.panel_nuevoTipo.Enabled = true;
            this.panel_tipos.Enabled = true;
        }

        private void button_aceptarModifTipo_Click(object sender, EventArgs e)
        {
            try
            {
                if (textBox_modifTipo.Text.Length == 0)
                {
                    MessageBox.Show("Debe escribir un nombre para la modificación", "Modificar Tipo", MessageBoxButtons.OK, MessageBoxIcon.Warning);
                }
                else{
                //comprovem si el nom de la nova secció és nou
                if (client.BuscarTipo(textBox_modifTipo.Text) == true)
                {
                    //la secció ja existeix i no es pot fer l'alta
                    MessageBox.Show("Esta tipo ya existe. No se puede efectuar la modificación", "Modificar Tipo", MessageBoxButtons.OK, MessageBoxIcon.Warning);
                    textBox_modifTipo.Clear();
                }
                else
                {
                    //recuperem el tipus a modificar
                    int i = listBox_tipos.SelectedIndex;
                    Tipo tipoModif = tipos.ElementAt(i);

                    client.ModificarTipo(tipoModif.Id, textBox_modifTipo.Text);
                    cargarListaTipos(); //actualitzem la llista de tipus
                        this.panel_modifTipo.Visible = false;
                        this.panel_nuevoTipo.Enabled = true;
                        this.panel_tipos.Enabled = true;
                
                }
                }
            }
            catch (NullReferenceException ex)
            {
                MessageBox.Show(ex.Message, "Tipos: " + ex.Source, MessageBoxButtons.OK, MessageBoxIcon.Error);
            }
            catch (SystemException ex1)
            {
                MessageBox.Show(ex1.Message, "Tipos: " + ex1.Source, MessageBoxButtons.OK, MessageBoxIcon.Error);
            }
            catch (Exception ex2)
            {
                MessageBox.Show(ex2.Message, "Tipos: " + ex2.Source, MessageBoxButtons.OK, MessageBoxIcon.Error);
            }
        }
       

        
        
    }
}
