﻿using System;
using System.Resources;
using System.Collections;
using System.Collections.Generic;
using System.Linq;
using System.Net.Mail;
using System.Text;
using System.Web;
using System.Web.UI;
using System.Web.UI.WebControls;
using System.Web.Security;
using SitioWeb.ServiceReference1;

namespace SitioWeb
{
    public partial class Reservas : System.Web.UI.Page
    {
        private ServiceReference1.Service1Client client; //variable per accedir al servei WCF
        private Cliente usuario;
        private List<Turno> turnos;
        private List<Mesa> mesas;
        private Mesa mesaReserva;
        private String url;

        protected void Page_Load(object sender, EventArgs e)
        {
            if (!Page.IsPostBack)
            {//codi que s'executa la 1ª vegada que es carrega la pàg
                client = new ServiceReference1.Service1Client();//nova instancia del servei WCF
                usuario = (Cliente)Session["Usuario"];
                url = (String)Session["url"];

                cargarComboTurnos();
                Calendar1.SelectedDate = DateTime.Now;
                DropDownList_turnos.SelectedIndex = 0;
                DropDownList_personas.SelectedIndex = 0;
                
            }
            else
            {//codi que s'executa cada vegada que es carrega la pàg
                client = new ServiceReference1.Service1Client();//nova instancia del servei WCF
                usuario = (Cliente)Session["Usuario"];
                url = (String)Session["url"];
                OkMessage.Text = "";
            }
            
        }

        protected void Page_UnLoad(object sender, EventArgs e)
        {
            Session["url"] = "~/Reservas.aspx";
        }

        private void cargarComboTurnos()
        {
                //netejem la llista per si te dades
                this.DropDownList_turnos.Items.Clear();
                //recuperem els torns de la BBDD
                turnos = client.ListarTurnos().ToList();
                for (int i = 0; i < turnos.Count(); i++)
                {//Carreguem la llista de torns
                    this.DropDownList_turnos.Items.Add(turnos.ElementAt(i).Hora.ToString());
                }
        }

        

        protected void Button1_Click(object sender, EventArgs e)
        {
            DateTime selectedDate = Calendar1.SelectedDate.Date;
            bool festivo = client.BuscarFestivo(selectedDate);

            
            
            if (selectedDate < DateTime.Now.Date)
            {
                ErrorMessage.Text = "No se puede seleccionar una fecha previa a la de hoy";
            }
            else if (festivo == true)
            {
                ErrorMessage.Text = "Lo sentimos. La fecha seleccionada es festiva";
            }
            else
            {
                cargarMesasLibres();
            }
        }

        //mètode per assignar una taula lliure a la reserva
        private void cargarMesasLibres()
        {
            
            //recuperem les taules lliures de la BBDD
            mesas = client.ListarMesasLibres(Calendar1.SelectedDate.Date, DropDownList_turnos.SelectedValue).ToList();
            if (mesas.Count() == 0) //comprovem que hi ha alguna taula lliure
            {
                ErrorMessage.Text = "Lo sentimos. No tenemos disponibilidad para la fecha y hora seleccionadas";
            }
            else
            {
                mesaReserva = asignarMesa(mesas);
                if (mesaReserva != null)
                {
                    Panel1.Enabled = false;
                    Button1.Enabled = false;
                    Panel3.Enabled = true;
                    Button_cancelar.Enabled = true;
                    Button_guardar.Enabled = true;

                    //es carreguen les dades de la pre-reserva:
                    TextBox_fechaR.Text = Calendar1.SelectedDate.Date.ToShortDateString();
                    TextBox_horaR.Text = DropDownList_turnos.SelectedValue;
                    TextBox_personasR.Text = DropDownList_personas.SelectedValue;
                    textbox_mesaR.Text = mesaReserva.numero.ToString();

                    //es carreguen les dades de l'usuari loguejat
                    if (usuario != null)
                    {
                        cargarDatosSesionUsuario(usuario);
                    }

                }
                else { ErrorMessage.Text = "Lo sentimos. No tenemos mesa libre para el nº de comensales solicitado"; }
                        
            
            }

        }


        //mètode per buscar una taula lliure adequada al nº de comensals
        protected Mesa asignarMesa(List<Mesa> mesas) 
        {
            int comensales = int.Parse(DropDownList_personas.SelectedValue);
            Mesa mesaAsignada = null;
            int i = 0;
            while (mesaAsignada == null && i <mesas.Count()){
                //busquem la taula adequada al nº de persones                    
                if (comensales <= mesas.ElementAt(i).comensales)
                {
                    mesaAsignada = new Mesa();
                    mesaAsignada = mesas.ElementAt(i);//s'ha trobat una taula 
                }
                else { i++; }
            }
            return mesaAsignada;
        }


        protected void Calendar1_SelectionChanged(object sender, EventArgs e)
        {
            ErrorMessage.Text = "";

        }
        protected void DropDownList_turnos_SelectedIndexChanged(object sender, EventArgs e)
        {
            ErrorMessage.Text = "";
        }

        protected void DropDownList_personas_SelectedIndexChanged(object sender, EventArgs e)
        {
            ErrorMessage.Text = "";
        }

        protected void Button_cancelar_Click(object sender, EventArgs e)
        {
            Panel1.Enabled = true;
            Button1.Enabled = true;
            Panel3.Enabled = false;
            Button_cancelar.Enabled = false;
            Button_guardar.Enabled = false;
            
            //netejem les dades del panel
            TextBox_fechaR.Text = "";
            TextBox_horaR.Text = "";
            TextBox_personasR.Text = "";
            textbox_mesaR.Text = "";

        }

        protected void Button_guardar_Click(object sender, EventArgs e)
        {
            //nova reserva
            int nuevaReserva = client.AltaReserva(int.Parse(codCliente.Text), TextBox_nombreR.Text, TextBox_telefR.Text, TextBoxmailR.Text,
                int.Parse(TextBox_personasR.Text), "confirmada", "online", DateTime.Parse(TextBox_fechaR.Text), int.Parse(textbox_mesaR.Text), TextBox_horaR.Text);

            OkMessage.Text = "Reserva realizada con éxito. Recibirá un Email de confirmación" + "\n"+"Su número de reserva es: "+nuevaReserva;
           
            //envio eMail confirmació de la reserva:
            //recuperem les dades del nostre servidor
            string smtpEmail = SitioWeb.Resources.eMail;
            string smtpPw = SitioWeb.Resources.password;
            string smtpServer = SitioWeb.Resources.ServidorSmtp;

            //creem el missatge de mail
            MailMessage correo = new MailMessage();
            correo.From = new MailAddress(smtpEmail);
            correo.To.Add(this.TextBoxmailR.Text);
            correo.Subject = "Confirmación Reserva, UOC Restaurant";

            StringBuilder sb = new StringBuilder();
            sb = sb.Append("Confirmación de Reserva:");
            sb = sb.Append("\n");
            sb = sb.Append("Su reserva nº: " + nuevaReserva);
            sb = sb.Append("\n");
            sb = sb.Append("para el día: " + TextBox_fechaR.Text + " - " + TextBox_horaR.Text);
            sb = sb.Append("\n");
            sb = sb.Append("Su mesa asignada es: "+ textbox_mesaR.Text + " para: "+ TextBox_personasR.Text + " comensales");
            sb = sb.Append("\n" + "\n");
            sb = sb.Append("Gracias por su reserva");
            sb = sb.Append("\n");
            sb = sb.Append("UOC Restaurant");

            correo.Body = sb.ToString();
            correo.IsBodyHtml = false;
            correo.Priority = MailPriority.Normal;
            SmtpClient smtp = new SmtpClient(smtpServer);
            smtp.EnableSsl = true;
            smtp.UseDefaultCredentials = false;
            smtp.Port = 25;
            smtp.Credentials = new System.Net.NetworkCredential(smtpEmail, smtpPw);

            try
            {//enviem el missatge
                smtp.Send(correo);
            }
            catch (Exception ex)
            {
                ErrorMessage.Text = ex.ToString();
            }


            //netejar dades
            Panel1.Enabled = true;
            Button1.Enabled = true;
            Panel3.Enabled = false;
            Button_cancelar.Enabled = false;
            Button_guardar.Enabled = false;
            //netejem les dades del panel
            TextBox_fechaR.Text = "";
            TextBox_horaR.Text = "";
            TextBox_personasR.Text = "";
            textbox_mesaR.Text = "";
            
        }


        protected void cargarDatosSesionUsuario(Cliente identificado)
        {
            codCliente.Text = identificado.Id.ToString();
            TextBox_nombreR.Text = identificado.Nombre;
            TextBoxmailR.Text = identificado.Mail;
            TextBox_telefR.Text = identificado.Telefono;
        }
        

    }
}