﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Linq.Expressions;
using System.Data;
using System.Data.Linq;
using System.Runtime.Serialization;
using System.ServiceModel;
using System.Text;
using AccesoBDRest;


namespace WcfService
{
    // NOTA: puede usar el comando "Rename" del menú "Refactorizar" para cambiar el nombre de clase "Service1" en el código y en el archivo de configuración a la vez.
    public class Service1 : IService1
    {
        #region Miembros de IService

        //atribut per accedir a la BBDD
        private DataClassesRestDataContext db;

        //Mètode per obrir la conexió amb la BBDD
        private void ConectarBBDD()
        {
            try
            {
                db = new DataClassesRestDataContext();
                db.Connection.Open();
            }            
            catch (SystemException)
            {
                throw;
            }
            catch (Exception)
            {
                throw;
            }
        }

        //Mètode per tancar la conexió amb la BBDD
        private void DesconectarBBDD()
        {
            try
            {
               db.Connection.Close();
            }            
            catch (SystemException)
            {
                throw;
            }
            catch (Exception)
            {
                throw;
            }
        }

       
        //********* Mètodes per gestionar els DIES FESTIUS ************

        //mètode per afegir un dia festiu
        public void AltaFestivo(DateTime pFecha)
        {
            try
            {
                ConectarBBDD();//obrim la connexió
                Festivo f = new Festivo();
                f.Fecha = pFecha;
                db.Festivo.InsertOnSubmit(f);
                db.SubmitChanges();
                DesconectarBBDD();//tanquem la connexió
            }
            catch (NullReferenceException)
            {
                throw;
            }
            catch (SystemException)
            {
                throw;
            }
            catch (Exception)
            {
                throw;
            }
        }

        

        //mètode per eliminar un festiu
        public void EliminaFestivo(DateTime pFecha)
        {
            try
            {
                ConectarBBDD();
                Festivo fecha = db.Festivo.Single(f => f.Fecha == pFecha);

                if (fecha != null)
                {
                    db.Festivo.DeleteOnSubmit(fecha);
                    db.SubmitChanges();
                    DesconectarBBDD();
                }
                else
                {
                    DesconectarBBDD();
                }
            }
            catch (NullReferenceException)
            {
                throw;
            }
            catch (SystemException)
            {
                throw;
            }
            catch (Exception)
            {
                throw;
            }
        }

        //mètode per buscar el festiu passat com a paràmetre
        public Boolean BuscarFestivo(DateTime pFecha)
        {
            try
            {
                ConectarBBDD();

                //Consulta si existeix un festiu igual al rebut com a paràmetre.
                var result = from f in db.Festivo
                             where f.Fecha == pFecha
                             select f;
                if (result.Count() != 0) //Si existeix algun registre es que hem trobat el festiu.
                {
                    DesconectarBBDD();
                    return true;
                }
                else // no s'ha trobat el festiu
                {
                    DesconectarBBDD();
                    return false;
                }
            }
            catch (NullReferenceException)
            {
                throw;
            }
            catch (SystemException)
            {
                throw;
            }
            catch (Exception)
            {
                throw;
            }
        }

        //método que devuelve la lista de festivos de la BBDD
        public List<Festivo> ListarFestivos()
        {
            try
            {
                List<Festivo> listaFestivos = new List<Festivo>();
                //obrim la connexió
                ConectarBBDD();
                var festivos = (from f in db.Festivo orderby f.Fecha select f).ToList();
                //tanquem la connexió
                DesconectarBBDD();
                foreach (var f in festivos)
                {
                    Festivo nuevoFestivo = new Festivo();
                    nuevoFestivo.Fecha = f.Fecha;
                    listaFestivos.Add(nuevoFestivo);
                }
                return listaFestivos;
            }
            catch (NullReferenceException)
            {
                throw;
            }
            catch (SystemException)
            {
                throw;
            }
            catch (Exception)
            {
                throw;
            }

        }

        


        //********* Mètodes per gestionar els TORNS ************

        public List<Turno> ListarTurnos()
        {
            try
            {
                List<Turno> listaTurnos = new List<Turno>();
                //obrim la connexió
                ConectarBBDD();
                var turnos = (from t in db.Turno orderby t.Hora select t).ToList();
                //tanquem la connexió
                DesconectarBBDD();
                foreach (var t in turnos)
                {
                    Turno nuevoTurno = new Turno();
                    nuevoTurno.Hora = t.Hora;
                    listaTurnos.Add(nuevoTurno);
                }
                return listaTurnos;
            }
            catch (NullReferenceException)
            {
                throw;
            }
            catch (SystemException)
            {
                throw;
            }
            catch (Exception)
            {
                throw;
            }
            
        }

        

        //mètode per afegir un nou torn
        public void AltaTurno(string pHora)
        {
            try
            {
                ConectarBBDD();//obrim la connexió
                Turno t = new Turno();
                t.Hora = pHora;
                db.Turno.InsertOnSubmit(t);
                db.SubmitChanges();
                DesconectarBBDD();//tanquem la connexió
            }
            catch (NullReferenceException)
            {
                throw;
            }
            catch (SystemException)
            {
                throw;
            }
            catch (Exception)
            {
                throw;
            }
        }

        //mètode per buscar el torn passat com a paràmetre
        public Boolean BuscarTurno(String pHora)
        {
            try
            {
                ConectarBBDD();

                //Consulta si existeix una taula amb numTaula igual a la rebuda com a paràmetre.
                var result = from t in db.Turno
                             where t.Hora == pHora
                             select t;
                if (result.Count() != 0) //Si existeix algun registre es que hem trobat la taula.
                {
                    DesconectarBBDD();
                    return true;
                }
                else // no s'ha trobat la taula amb aquest núm.
                {
                    DesconectarBBDD();
                    return false;
                }
            }
            catch (NullReferenceException)
            {
                throw;
            }
            catch (SystemException)
            {
                throw;
            }
            catch (Exception)
            {
                throw;
            }
        }



        //********* Mètodes per gestionar les TAULES ************

        public List<Mesa> ListarMesas()
        {
            try
            {
                List<Mesa> listaMesas = new List<Mesa>();
                //obrim la connexió
                ConectarBBDD();
                var mesas = (from m in db.Mesa orderby m.numero select m);
                //tanquem la connexió
                DesconectarBBDD();
                foreach (var m in mesas)
                {
                    Mesa nuevaMesa = new Mesa();
                    nuevaMesa.numero = m.numero;
                    nuevaMesa.comensales = m.comensales;
                    listaMesas.Add(nuevaMesa);
                }
                return listaMesas;
            }
            catch (NullReferenceException)
            {
                throw;
            }
            catch (SystemException)
            {
                throw;
            }
            catch (Exception)
            {
                throw;
            }
        }

        //mètode per afegir una nova taula
        public void AltaMesa(int pNumero, int pComensales)
        {
            try
            {
                ConectarBBDD();//obrim la connexió
                Mesa m = new Mesa();
                m.numero = pNumero;
                m.comensales = pComensales;
                db.Mesa.InsertOnSubmit(m);
                db.SubmitChanges();
                DesconectarBBDD();//tanquem la connexió
            }
            catch (NullReferenceException)
            {
                throw;
            }
            catch (SystemException)
            {
                throw;
            }
            catch (Exception)
            {
                throw;
            }
        }

        public Boolean BuscarMesa(int numMesa)
        {
            try
            {
                ConectarBBDD();

                //Consulta si existeix una taula amb numTaula igual a la rebuda com a paràmetre.
                var result = from m in db.Mesa
                             where m.numero == numMesa
                             select m;
                if (result.Count() != 0) //Si existeix algun registre es que hem trobat la taula.
                {
                    DesconectarBBDD();
                    return true;
                }
                else // no s'ha trobat la taula amb aquest núm.
                {
                    DesconectarBBDD();
                    return false;
                }
            }
            catch (NullReferenceException)
            {
                throw;
            }
            catch (SystemException)
            {
                throw;
            }
            catch (Exception)
            {
                throw;
            }
        }

        //mètode per trobar les taules lliures en una data i un torn
        public List<Mesa> ListarMesasLibres(DateTime fecha, string turno)
        {
            
            try
            {
                List<Mesa> listaMesas= new List<Mesa>();
                //obrim la connexió
                ConectarBBDD();
                var mesas = (from m in db.Mesa
                             where !(from a in db.Aforo
                                     where a.Fecha == fecha && a.Turno == turno
                                     select a.Mesa).Contains(m.numero)
                             orderby m.comensales select m);
                //tanquem la connexió
                DesconectarBBDD();
                foreach (var m in mesas)
                    {
                        Mesa nuevaMesa = new Mesa();
                        nuevaMesa.numero = m.numero;
                        nuevaMesa.comensales = m.comensales;
                        listaMesas.Add(nuevaMesa);
                    }
                return listaMesas;
            }
            catch (NullReferenceException)
            {
                throw;
            }
            catch (SystemException)
            {
                throw;
            }
            catch (Exception)
            {
                throw;
            }
        }

        //********* Mètodes per gestionar les SECCIONS ************

        public List<Seccion> ListarSecciones()
        {
            try
            {
                List<Seccion> listaSecciones = new List<Seccion>();

                //obrim la connexió
                ConectarBBDD();
                var secciones = (from s in db.Seccion orderby s.nombre select s);
                //tanquem la connexió
                DesconectarBBDD();

                foreach (var s in secciones)
                {
                    Seccion nuevaSeccion = new Seccion();
                    nuevaSeccion.Id = s.Id;
                    nuevaSeccion.nombre = s.nombre;
                    listaSecciones.Add(nuevaSeccion);
                }
                return listaSecciones;
            }
            catch (NullReferenceException)
            {
                throw;
            }
            catch (SystemException)
            {
                throw;
            }
            catch (Exception)
            {
                throw;
            }
        }


        //mètode per afegir una secció
        public void AltaSeccion(string nombreSeccion)
        {
            try
            {
                ConectarBBDD();

                Seccion s = new Seccion();
                s.nombre = nombreSeccion;
                
                db.Seccion.InsertOnSubmit(s);
                db.SubmitChanges();

                DesconectarBBDD();
            }
            catch (NullReferenceException)
            {
                throw;
            }
            catch (SystemException)
            {
                throw;
            }
            catch (Exception)
            {
                throw;
            }
        }


        //mètode per eliminar una secció
        public void EliminarSeccion(int idSeccion)
        {
            try
            {
                ConectarBBDD();
                Seccion seccion = db.Seccion.Single(s => s.Id == idSeccion);

                if (seccion != null)
                {
                    db.Seccion.DeleteOnSubmit(seccion);
                    db.SubmitChanges();
                    DesconectarBBDD();
                }
                else
                {
                    DesconectarBBDD();
                }
            }
            catch (NullReferenceException)
            {
                throw;
            }
            catch (SystemException)
            {
                throw;
            }
            catch (Exception)
            {
                throw;
            }
        }

        //mètode per modificar el nom d'una secció
        public void ModificarSeccion(int idSeccion, string newNombre)
        {
            try
            {
                ConectarBBDD();
                Seccion seccion = db.Seccion.Single(s => s.Id == idSeccion);
                if (seccion != null)
                {
                    seccion.nombre = newNombre;
                    db.SubmitChanges();
                    DesconectarBBDD();
                }
                else
                {
                    DesconectarBBDD();
                }
            }
            catch (NullReferenceException)
            {
                throw;
            }
            catch (SystemException)
            {
                throw;
            }
            catch (Exception)
            {
                throw;
            }
        }

        //Consulta si existeix una secció amb nomSecció igual a la rebuda com a paràmetre.
        public Boolean BuscarSeccion(String nombreSeccion)
        {
            try
            {
                ConectarBBDD();
                
                var result = from s in db.Seccion
                             where s.nombre == nombreSeccion
                             select s;
                if (result.Count() != 0) //Si existeix algun registre es que hem trobat la secció.
                {
                    DesconectarBBDD();
                    return true;
                }
                else // no s'ha trobat la secció amb aquest nom.
                {
                    DesconectarBBDD();
                    return false;
                }
            }
            catch (NullReferenceException)
            {
                throw;
            }
            catch (SystemException)
            {
                throw;
            }
            catch (Exception)
            {
                throw;
            }
        }


        //Consulta si existeix una secció inclosa en algun plat.
        public Boolean BuscarSeccionEnPlato(int idSeccion)
        {
            try
            {
                ConectarBBDD();
                var result = from p in db.Plato
                             where p.Seccion == idSeccion
                             select p;
                if (result.Count() != 0) //Si existeix algun registre es que hem trobat la secció.
                {
                    DesconectarBBDD();
                    return true;
                }
                else // no s'ha trobat la secció           
                {
                    DesconectarBBDD();
                    return false;
                }
            }
            catch (NullReferenceException)
            {
                throw;
            }
            catch (SystemException)
            {
                throw;
            }
            catch (Exception)
            {
                throw;
            }
        }


        public string nombreSeccion(int idSeccion)
        {
            try
            {
                ConectarBBDD();
                Seccion seccion = db.Seccion.Single(s => s.Id == idSeccion);
                if (seccion != null)
                {
                    DesconectarBBDD();
                    return seccion.nombre;
                }
                else
                {
                    DesconectarBBDD();
                    return "";
                }
            }
            catch (NullReferenceException)
            {
                throw;
            }
            catch (SystemException)
            {
                throw;
            }
            catch (Exception)
            {
                throw;
            }
        }
            


        //********* Mètodes per gestionar els PLATS ************

        public List<Plato> ListarPlatos()
        {
            try
            {
                List<Plato> listaPlatos = new List<Plato>();

                //obrim la connexió
                ConectarBBDD();
                var platos = (from p in db.Plato
                              orderby p.Idioma1
                              select p);

                //tanquem la connexió
                DesconectarBBDD();

                foreach (var p in platos)
                {
                    Plato nuevoPlato = new Plato();
                    nuevoPlato.Id = p.Id;
                    nuevoPlato.Idioma1 = p.Idioma1;
                    nuevoPlato.Idioma2 = p.Idioma2;
                    nuevoPlato.Idioma3 = p.Idioma3;
                    nuevoPlato.Precio = p.Precio;
                    nuevoPlato.Seccion = p.Seccion;
                    nuevoPlato.MinComensales = p.MinComensales;
                    nuevoPlato.Iva = p.Iva;
                    nuevoPlato.Disponible = p.Disponible;
                    listaPlatos.Add(nuevoPlato);

                }
                return listaPlatos;

            }
            catch (NullReferenceException)
            {
                throw;
            }
            catch (SystemException)
            {
                throw;
            }
            catch (Exception)
            {
                throw;
            }
        }


        //mètode per llistar els plats d'una secció
        public List<Plato> ListarPlatosSeccion(int numSeccion)
        {
            try
            {
                List<Plato> listaPlatos = new List<Plato>();

                //obrim la connexió
                ConectarBBDD();
                var platos = (from p in db.Plato 
                              where (p.Seccion == numSeccion) 
                                orderby p.Idioma1 select p);
                //tanquem la connexió
                DesconectarBBDD();

                foreach (var p in platos)
                {
                    Plato nuevoPlato = new Plato();
                    nuevoPlato.Id = p.Id;
                    nuevoPlato.Idioma1 = p.Idioma1;
                    nuevoPlato.Idioma2 = p.Idioma2;
                    nuevoPlato.Idioma3 = p.Idioma3;
                    nuevoPlato.Precio = p.Precio;
                    nuevoPlato.Seccion = p.Seccion;
                    nuevoPlato.MinComensales = p.MinComensales;
                    nuevoPlato.Iva = p.Iva;
                    nuevoPlato.Disponible = p.Disponible;
                    listaPlatos.Add(nuevoPlato);
                }
                return listaPlatos;
            }
            catch (NullReferenceException)
            {
                throw;
            }
            catch (SystemException)
            {
                throw;
            }
            catch (Exception)
            {
                throw;
            }
        }


        //mètode per llistar els plats que contenen la paraula passada com a paràmetre
        public List<Plato> ListarPlatosNombre(string nombre)
        {
            try
            {
                List<Plato> listaPlatos = new List<Plato>();

                //obrim la connexió
                ConectarBBDD();
                var platos = (from p in db.Plato
                              where (p.Idioma1.Contains(nombre))
                              orderby p.Idioma1
                              select p);
                //tanquem la connexió
                DesconectarBBDD();

                foreach (var p in platos)
                {
                    Plato nuevoPlato = new Plato();
                    nuevoPlato.Id = p.Id;
                    nuevoPlato.Idioma1 = p.Idioma1;
                    nuevoPlato.Idioma2 = p.Idioma2;
                    nuevoPlato.Idioma3 = p.Idioma3;
                    nuevoPlato.Precio = p.Precio;
                    nuevoPlato.Seccion = p.Seccion;
                    nuevoPlato.MinComensales = p.MinComensales;
                    nuevoPlato.Iva = p.Iva;
                    nuevoPlato.Disponible = p.Disponible;
                    listaPlatos.Add(nuevoPlato);
                }
                return listaPlatos;
            }
            catch (NullReferenceException)
            {
                throw;
            }
            catch (SystemException)
            {
                throw;
            }
            catch (Exception)
            {
                throw;
            }
        }

        //mètode per afegir un plat
        public void AltaPlato(string pIdioma1, string pIdioma2, string pIdioma3, float pPrecio, int pSeccion, int pMinComen, bool pIva, bool pDisp)
        {
            try
            {
                ConectarBBDD();

                Plato p = new Plato();
                p.Idioma1 = pIdioma1;
                p.Idioma2 = pIdioma2;
                p.Idioma3 = pIdioma3;
                p.Precio = pPrecio;
                p.Seccion = pSeccion;
                p.MinComensales = pMinComen;
                p.Iva = pIva;
                p.Disponible = pDisp;

                db.Plato.InsertOnSubmit(p);
                db.SubmitChanges();

                DesconectarBBDD();
            }
            catch (NullReferenceException)
            {
                throw;
            }
            catch (SystemException)
            {
                throw;
            }
            catch (Exception)
            {
                throw;
            }
        }

        //mètode per modificar un plat
        public void ModificarPlato(int idPlato, string pIdioma1, string pIdioma2, string pIdioma3, float pPrecio, int pSeccion, int pMinComen, bool pIva, bool pDisp)
        {
            try
            {
                ConectarBBDD();
                Plato plato = db.Plato.Single(p => p.Id == idPlato);

                if (plato != null)
                {
                    plato.Idioma1 = pIdioma1;
                    plato.Idioma2 = pIdioma2;
                    plato.Idioma3 = pIdioma3;
                    plato.Precio = pPrecio;
                    plato.Seccion = pSeccion;
                    plato.MinComensales = pMinComen;
                    plato.Iva = pIva;
                    plato.Disponible = pDisp;
                    db.SubmitChanges();
                    DesconectarBBDD();
                }
                else
                {
                    DesconectarBBDD();
                }
            }
            catch (NullReferenceException)
            {
                throw;
            }
            catch (SystemException)
            {
                throw;
            }
            catch (Exception)
            {
                throw;
            }
        }

        public void EliminarPlato(int idPlato)
        {
            try
            {
                ConectarBBDD();
                Plato plato = db.Plato.Single(p => p.Id == idPlato);

                if (plato != null)
                {
                    db.Plato.DeleteOnSubmit(plato);
                    db.SubmitChanges();
                    DesconectarBBDD();
                }
                else
                {
                    DesconectarBBDD();
                }
            }
            catch (NullReferenceException)
            {
                throw;
            }
            catch (SystemException)
            {
                throw;
            }
            catch (Exception)
            {
                throw;
            }
        }

        //Consulta si existeix un plat inclos en algun menu o alguna carta.
        public Boolean BuscarPlatoEnMenuCarta(int idPlato)
        {
            try
            {
                ConectarBBDD();
                var result1 = from pm in db.PlatosMenu
                             where pm.Plato == idPlato
                             select pm;
                var result2 = from pc in db.PlatosCarta
                              where pc.Plato == idPlato
                              select pc;
                if ((result1.Count() != 0) || (result2.Count() != 0))//Si existeix algun registre es que hem trobat el plat.
                {
                    DesconectarBBDD();
                    return true;
                }
                else // no s'ha trobat la secció           
                {
                    DesconectarBBDD();
                    return false;
                }
            }
            catch (NullReferenceException)
            {
                throw;
            }
            catch (SystemException)
            {
                throw;
            }
            catch (Exception)
            {
                throw;
            }
        }


        //********* Mètodes per gestionar els TIPUS de PLATS ************

        public List<Tipo> ListarTipos()
        {
            try
            {
                List<Tipo> listaTipos = new List<Tipo>();
                //obrim la connexió
                ConectarBBDD();
                var tipos = (from t in db.Tipo orderby t.Id select t);
                //tanquem la connexió
                DesconectarBBDD();

                foreach (var t in tipos)
                {
                    Tipo nuevoTipo = new Tipo();
                    nuevoTipo.Id = t.Id;
                    nuevoTipo.Tipo1 = t.Tipo1;
                    listaTipos.Add(nuevoTipo);
                }
                return listaTipos;
            }
            catch (NullReferenceException)
            {
                throw;
            }
            catch (SystemException)
            {
                throw;
            }
            catch (Exception)
            {
                throw;
            }
        }

        public void AltaTipo(string pTipo)
        {
            try
            {
                ConectarBBDD();

                Tipo t = new Tipo();
                t.Tipo1 = pTipo;

                db.Tipo.InsertOnSubmit(t);
                db.SubmitChanges();

                DesconectarBBDD();
            }
            catch (NullReferenceException)
            {
                throw;
            }
            catch (SystemException)
            {
                throw;
            }
            catch (Exception)
            {
                throw;
            }
        }

        //mètode per eliminar un tipus
        public void EliminarTipo(int idTipo)
        {
            try
            {
                ConectarBBDD();
                Tipo tipo = db.Tipo.Single(t => t.Id == idTipo);

                if (tipo != null)
                {
                    db.Tipo.DeleteOnSubmit(tipo);
                    db.SubmitChanges();
                    DesconectarBBDD();
                }
                else
                {
                    DesconectarBBDD();
                }
            }
            catch (NullReferenceException)
            {
                throw;
            }
            catch (SystemException)
            {
                throw;
            }
            catch (Exception)
            {
                throw;
            }
        }

        //mètode per modificar el nom d'un tipus
        public void ModificarTipo(int idTipo, string newNombre)
        {
            try
            {
                ConectarBBDD();
                Tipo tipo = db.Tipo.Single(t => t.Id == idTipo);

                if (tipo != null)
                {
                    tipo.Tipo1 = newNombre;
                    db.SubmitChanges();
                    DesconectarBBDD();
                }
                else
                {
                    DesconectarBBDD();
                }
            }
            catch (NullReferenceException)
            {
                throw;
            }
            catch (SystemException)
            {
                throw;
            }
            catch (Exception)
            {
                throw;
            }
        }

        public Boolean BuscarTipo(String nombreTipo)
        {
            try
            {
                ConectarBBDD();

                //Consulta si existeix tipus amb nomTipus igual al rebutcom a paràmetre.
                var result = from t in db.Tipo
                             where t.Tipo1 == nombreTipo
                             select t;
                if (result.Count() != 0) //Si existeix algun registre es que hem trobat el tipus.
                {
                    DesconectarBBDD();
                    return true;
                }
                else // no s'ha trobat la secció amb aquest nom.
                {
                    DesconectarBBDD();
                    return false;
                }
            }
            catch (NullReferenceException)
            {
                throw;
            }
            catch (SystemException)
            {
                throw;
            }
            catch (Exception)
            {
                throw;
            }
        }

        //Consulta si existeix un tipus inclos en algun menu.
        public Boolean BuscarTipoEnMenu(int idTipo)
        {
            try
            {
                ConectarBBDD();
                var result = from pm in db.PlatosMenu
                             where pm.Tipo == idTipo
                             select pm;
                if (result.Count() != 0) //Si existeix algun registre es que hem trobat la secció.
                {
                    DesconectarBBDD();
                    return true;
                }
                else // no s'ha trobat la secció           
                {
                    DesconectarBBDD();
                    return false;
                }
            }
            catch (NullReferenceException)
            {
                throw;
            }
            catch (SystemException)
            {
                throw;
            }
            catch (Exception)
            {
                throw;
            }
        }


        public string nombreTipo(int idTipo)
        {
            try
            {
                ConectarBBDD();
                Tipo tipo = db.Tipo.Single(s => s.Id == idTipo);
                if (tipo != null)
                {
                    DesconectarBBDD();
                    return tipo.Tipo1;
                }
                else
                {
                    DesconectarBBDD();
                    return "";
                }
            }
            catch (NullReferenceException)
            {
                throw;
            }
            catch (SystemException)
            {
                throw;
            }
            catch (Exception)
            {
                throw;
            }
        }
            


        //********* Mètodes per gestionar els MENUS ************

        public List<Menu> ListarMenus()
        {
            try
            {
                List<Menu> listaMenus = new List<Menu>();
                //obrim la connexió
                ConectarBBDD();
                var menus = (from m in db.Menu select m);
                //tanquem la connexió
                DesconectarBBDD();
                foreach (var m in menus)
                {
                    Menu nuevoMenu = new Menu();
                    nuevoMenu.Id = m.Id;
                    nuevoMenu.Fecha = m.Fecha;
                    nuevoMenu.Nombre = m.Nombre;
                    nuevoMenu.Precio = m.Precio;
                    nuevoMenu.MinComensales = m.MinComensales;
                    nuevoMenu.Bebida = m.Bebida;
                    nuevoMenu.Iva = m.Iva;
                    nuevoMenu.Publicar = m.Publicar;
                    listaMenus.Add(nuevoMenu);
                }
                return listaMenus;
            }
            catch (NullReferenceException)
            {
                throw;
            }
            catch (SystemException)
            {
                throw;
            }
            catch (Exception)
            {
                throw;
            }
        }



        //mètode per afegir un menú i que retorna l'id del nou menu
        public int AltaMenu(DateTime pDate, string pNombre, float pPrecio, int pMinComen, bool pBebida, bool pIva, bool pPublicar)
        {
            try
            {
                ConectarBBDD();

                Menu m = new Menu();
                m.Fecha = pDate;
                m.Nombre = pNombre;
                m.Precio = pPrecio;
                m.MinComensales = pMinComen;
                m.Bebida = pBebida;
                m.Iva = pIva;
                m.Publicar = pPublicar;
                
                db.Menu.InsertOnSubmit(m);
                db.SubmitChanges();

                DesconectarBBDD();
                return m.Id;
            }
            catch (NullReferenceException)
            {
                throw;
            }
            catch (SystemException)
            {
                throw;
            }
            catch (Exception)
            {
                throw;
            }
        }

        //mètode per modificar un menu
        public void ModificarMenu(int idMenu, DateTime pDate, string pNombre, float pPrecio, int pMinComen, bool pBebida, bool pIva)
        {
            try
            {
                ConectarBBDD();
                Menu menu = db.Menu.Single(m => m.Id == idMenu);
                if (menu != null)
                {
                    menu.Nombre = pNombre;
                    menu.Fecha = pDate;
                    menu.Precio = pPrecio;
                    menu.MinComensales = pMinComen;
                    menu.Bebida = pBebida;
                    menu.Iva = pIva;
                    db.SubmitChanges();
                    DesconectarBBDD();
                }
                else
                {
                    DesconectarBBDD();
                }
            }
            catch (NullReferenceException)
            {
                throw;
            }
            catch (SystemException)
            {
                throw;
            }
            catch (Exception)
            {
                throw;
            }
        }

        //mètode per eliminar un menu i els seus plats. Al eliminar el menu els plats del 
        //menu PlatosMoenu s'eliminen en cascada
        public void EliminarMenu(int idMenu)
        {
            try
            {
                ConectarBBDD();
                Menu menu = db.Menu.Single(m => m.Id == idMenu);

                if (menu != null)
                {
                    db.Menu.DeleteOnSubmit(menu);
                    db.SubmitChanges();
                    DesconectarBBDD();
                }
                else
                {
                    DesconectarBBDD();
                }
            }
            catch (NullReferenceException)
            {
                throw;
            }
            catch (SystemException)
            {
                throw;
            }
            catch (Exception)
            {
                throw;
            }
        }

        //mètode per afegir un plat a un menu
        public void AñadirPlatoMenu(int idMenu, int idPlato, int idTipo)
        {
            try
            {
                ConectarBBDD();

                PlatosMenu pm = new PlatosMenu();
                pm.Menu = idMenu;
                pm.Plato = idPlato;
                pm.Tipo = idTipo;

                db.PlatosMenu.InsertOnSubmit(pm);
                db.SubmitChanges();

                DesconectarBBDD();
            }
            catch (NullReferenceException)
            {
                throw;
            }
            catch (SystemException)
            {
                throw;
            }
            catch (Exception)
            {
                throw;
            }
        }

       


        //mètode per eliminar els plats d'un menú 
        public void EliminarPlatosMenu(int idMenu)
        {
            try
            {
                //obrim la connexió
                ConectarBBDD();
                var platos = (from p in db.PlatosMenu
                              where (p.Menu == idMenu)
                              select p);
                
                    foreach (var p in platos)
                    {
                        db.PlatosMenu.DeleteOnSubmit(p);
                        db.SubmitChanges();
                        DesconectarBBDD();
                    }
            }
            catch (NullReferenceException)
            {
                throw;
            }
            catch (SystemException)
            {
                throw;
            }
            catch (Exception)
            {
                throw;
            }
        }

        //mètode per llistar els plats que no estan al menú passat com a paràmetre
        public List<Plato> ListarPlDisponiblesMenu(int idMenu)
        {
            try
            {
                List<Plato> listaPlatos = new List<Plato>();

                //obrim la connexió
                ConectarBBDD();
                var platos = (from p in db.Plato
                              where !(from m in db.PlatosMenu where m.Menu == idMenu select m.Plato).Contains(p.Id)
                              orderby p.Idioma1
                              select p);

                //tanquem la connexió
                DesconectarBBDD();

                foreach (var p in platos)
                {
                    Plato nuevoPlato = new Plato();
                    nuevoPlato.Id = p.Id;
                    nuevoPlato.Idioma1 = p.Idioma1;
                    nuevoPlato.Idioma2 = p.Idioma2;
                    nuevoPlato.Idioma3 = p.Idioma3;
                    nuevoPlato.Precio = p.Precio;
                    nuevoPlato.Seccion = p.Seccion;
                    nuevoPlato.MinComensales = p.MinComensales;
                    nuevoPlato.Iva = p.Iva;
                    nuevoPlato.Disponible = p.Disponible;
                    listaPlatos.Add(nuevoPlato);
                }
                return listaPlatos;
            }
            catch (NullReferenceException)
            {
                throw;
            }
            catch (SystemException)
            {
                throw;
            }
            catch (Exception)
            {
                throw;
            }
        }

        //mètode per llistar els plats que no estan al menú passat com a paràmetre
        public List<Plato> ListarPlAsignadosMenu(int idMenu)
        {
            try
            {
                List<Plato> listaPlatos = new List<Plato>();

                //obrim la connexió
                ConectarBBDD();
                var platos = (from p in db.PlatosMenu
                              where (p.Menu == idMenu)
                              orderby p.Plato1.Idioma1
                              select p);
                //tanquem la connexió
                DesconectarBBDD();

                foreach (var p in platos)
                {
                    Plato nuevoPlato = new Plato();
                    nuevoPlato.Id = p.Plato1.Id;
                    nuevoPlato.Idioma1 = p.Plato1.Idioma1;
                    nuevoPlato.Idioma2 = p.Plato1.Idioma2;
                    nuevoPlato.Idioma3 = p.Plato1.Idioma3;
                    nuevoPlato.Precio = p.Plato1.Precio;
                    nuevoPlato.Seccion = p.Plato1.Seccion;
                    nuevoPlato.MinComensales = p.Plato1.MinComensales;
                    nuevoPlato.Iva = p.Plato1.Iva;
                    nuevoPlato.Disponible = p.Plato1.Disponible;
                    listaPlatos.Add(nuevoPlato);
                }
                return listaPlatos;
            }
            catch (NullReferenceException)
            {
                throw;
            }
            catch (SystemException)
            {
                throw;
            }
            catch (Exception)
            {
                throw;
            }
        }

        //mètode per buecar el tipus d'un plat d'un menu
        public int buscarTipoPlatoMenu(int idMenu, int idPlato)
        {
            try
            {
                //obrim la connexió
                ConectarBBDD();
                PlatosMenu pm = db.PlatosMenu.Single(s => s.Menu == idMenu && s.Plato == idPlato);
                if (pm != null)
                {
                    DesconectarBBDD();
                    return pm.Tipo;
                }
                else
                {
                    DesconectarBBDD();
                    return 0;
                }
            }
            catch (NullReferenceException)
            {
                throw;
            }
            catch (SystemException)
            {
                throw;
            }
            catch (Exception)
            {
                throw;
            }
        }

        
        //mètode per activar l'atribut boleà "publicar" de la carta passada per paràmetre
        public void PublicarMenu(int idMenu)
        {
            try
            {
                ConectarBBDD();
                Menu menu = db.Menu.Single(m => m.Id == idMenu);

                if (menu != null)
                {
                    menu.Publicar = true;
                    db.SubmitChanges();
                    DesconectarBBDD();
                }
                else
                {
                    DesconectarBBDD();
                }
            }

            catch (NullReferenceException)
            {
                throw;
            }
            catch (SystemException)
            {
                throw;
            }
            catch (Exception)
            {
                throw;
            }
        }

        //mètode per modificar l'atibut publicar de tots els menus i posar-los a fals
        public void DespublicarMenu()
        {
            try
            {
                ConectarBBDD();
                List<Menu> listaMenus = new List<Menu>();
                //obrim la connexió
                ConectarBBDD();
                var menus = (from m in db.Menu select m);

                foreach (var m in menus)
                {
                    m.Publicar = false;
                    db.SubmitChanges();
                    DesconectarBBDD();
                }
                DesconectarBBDD();
            }
            catch (NullReferenceException)
            {
                throw;
            }
            catch (SystemException)
            {
                throw;
            }
            catch (Exception)
            {
                throw;
            }
        }


        //mètode que retorna el Menu publicable
        public Menu menuWeb()
        {
            try
            {
                ConectarBBDD();
                Menu menu = db.Menu.Single(m => m.Publicar == true);

                if (menu != null)
                {
                    DesconectarBBDD();
                    Menu nouMenu = new Menu();
                    nouMenu.Id = menu.Id;
                    nouMenu.Nombre = menu.Nombre;
                    nouMenu.Iva = menu.Iva;
                    nouMenu.Precio = menu.Precio;
                    nouMenu.Bebida = menu.Bebida;
                    nouMenu.MinComensales = menu.MinComensales;
                    nouMenu.Fecha = menu.Fecha;
                    return nouMenu;
                }
                else
                {
                    DesconectarBBDD();
                    return null;
                }
            }

            catch (NullReferenceException)
            {
                throw;
            }
            catch (SystemException)
            {
                throw;
            }
            catch (Exception)
            {
                throw;
            }
        }


        public List<String> imprimirMenu()
        {
            try
            {
                List<String> platosMenu = new List<String>();

                //obrim la connexió
                ConectarBBDD();
                var reorderQuery = (from p in db.PlatosMenu
                              where (p.Menu1.Publicar == true)
                              group p by p.Tipo1.Tipo1 into reorderGroups 
                              select reorderGroups);
                //tanquem la connexió
                DesconectarBBDD();
                foreach (var stockGroup in reorderQuery)
                {
                    string texto1 = (stockGroup.Key).ToString();
                    platosMenu.Add(texto1);
                    platosMenu.Add(" ");
                    foreach (var plato in stockGroup)
                    {
                        string texto2 = plato.Plato1.Idioma1;                        
                        platosMenu.Add(texto2);
                        string texto3 = plato.Plato1.Idioma2;
                        platosMenu.Add(texto3);
                        string texto4 = plato.Plato1.Idioma3;
                        platosMenu.Add(texto4);
                    }
                    platosMenu.Add("--------------------------------------------------------------");
                }
                return platosMenu;
            }
            catch (NullReferenceException)
            {
                throw;
            }
            catch (SystemException)
            {
                throw;
            }
            catch (Exception)
            {
                throw;
            }
        }

       

        //********* Mètodes per gestionar les CARTES ************

        public List<Carta> ListarCartas()
        {
            try
            {
                List<Carta> listaCartas = new List<Carta>();
                //obrim la connexió
                ConectarBBDD();
                var cartas = (from c in db.Carta select c);
                //tanquem la connexió
                DesconectarBBDD();

                foreach (var c in cartas)
                {
                    Carta nuevaCarta = new Carta();
                    nuevaCarta.Id = c.Id;
                    nuevaCarta.Fecha = c.Fecha;
                    nuevaCarta.Nombre = c.Nombre;
                    nuevaCarta.Publicar = c.Publicar;
                    listaCartas.Add(nuevaCarta);
                }
                return listaCartas;
            }
            catch (NullReferenceException)
            {
                throw;
            }
            catch (SystemException)
            {
                throw;
            }
            catch (Exception)
            {
                throw;
            }
        }

        //mètode per afegir una carta i que retorna l'id de la nova carta
        public int AltaCarta(DateTime pDate, string pNombre, bool pPublicar)
        {
            try
            {
                ConectarBBDD();

                Carta c = new Carta();
                c.Fecha = pDate;
                c.Nombre = pNombre;               
                c.Publicar = pPublicar;

                db.Carta.InsertOnSubmit(c);
                db.SubmitChanges();

                DesconectarBBDD();
                return c.Id;
            }
            catch (NullReferenceException)
            {
                throw;
            }
            catch (SystemException)
            {
                throw;
            }
            catch (Exception)
            {
                throw;
            }
        }


        //mètode per modifcar un carta
        public void ModificarCarta(int idCarta, DateTime pDate, string pNombre)
        {
            try
            {
                ConectarBBDD();
                Carta carta = db.Carta.Single(c => c.Id == idCarta);
                if (carta != null)
                {
                    carta.Nombre = pNombre;
                    carta.Fecha = pDate;
                    db.SubmitChanges();
                    DesconectarBBDD();
                }
                else
                {
                    DesconectarBBDD();
                }
            }
            catch (NullReferenceException)
            {
                throw;
            }
            catch (SystemException)
            {
                throw;
            }
            catch (Exception)
            {
                throw;
            }
        }


        //mètode per afegir un plat a una carta
        public void AñadirPlatoCarta(int idCarta, int idPlato)
        {
            try
            {
                ConectarBBDD();

                PlatosCarta pc = new PlatosCarta();
                pc.Carta = idCarta;
                pc.Plato = idPlato;

                db.PlatosCarta.InsertOnSubmit(pc);
                db.SubmitChanges();

                DesconectarBBDD();
            }
            catch (NullReferenceException)
            {
                throw;
            }
            catch (SystemException)
            {
                throw;
            }
            catch (Exception)
            {
                throw;
            }
        }

        //mètode per llistar els plats que no estan a la carta passada com a paràmetre
        public List<Plato> ListarPlDisponiblesCarta(int idCarta)
        {
            try
            {
                List<Plato> listaPlatos = new List<Plato>();

                //obrim la connexió
                ConectarBBDD();
                var platos = (from p in db.Plato
                              where !(from m in db.PlatosCarta where m.Carta == idCarta select m.Plato).Contains(p.Id)
                              orderby p.Idioma1
                              select p);
                //tanquem la connexió
                DesconectarBBDD();

                foreach (var p in platos)
                {
                    Plato nuevoPlato = new Plato();
                    nuevoPlato.Id = p.Id;
                    nuevoPlato.Idioma1 = p.Idioma1;
                    nuevoPlato.Idioma2 = p.Idioma2;
                    nuevoPlato.Idioma3 = p.Idioma3;
                    nuevoPlato.Precio = p.Precio;
                    nuevoPlato.Seccion = p.Seccion;
                    nuevoPlato.MinComensales = p.MinComensales;
                    nuevoPlato.Iva = p.Iva;
                    nuevoPlato.Disponible = p.Disponible;
                    listaPlatos.Add(nuevoPlato);
                }
                return listaPlatos;
            }
            catch (NullReferenceException)
            {
                throw;
            }
            catch (SystemException)
            {
                throw;
            }
            catch (Exception)
            {
                throw;
            }
        }

        //mètode per llistar els plats que fomen part de la carta passada com a paràmetre
        public List<Plato> ListarPlAsignadosCarta(int idCarta)
        {
            try
            {
                List<Plato> listaPlatos = new List<Plato>();

                //obrim la connexió
                ConectarBBDD();
                var platos = (from p in db.PlatosCarta
                              where (p.Carta == idCarta)
                              orderby p.Plato1.Idioma1
                              select p);
                //tanquem la connexió
                DesconectarBBDD();

                foreach (var p in platos)
                {
                    Plato nuevoPlato = new Plato();
                    nuevoPlato.Id = p.Plato1.Id;
                    nuevoPlato.Idioma1 = p.Plato1.Idioma1;
                    nuevoPlato.Idioma2 = p.Plato1.Idioma2;
                    nuevoPlato.Idioma3 = p.Plato1.Idioma3;
                    nuevoPlato.Precio = p.Plato1.Precio;
                    nuevoPlato.Seccion = p.Plato1.Seccion;
                    nuevoPlato.MinComensales = p.Plato1.MinComensales;
                    nuevoPlato.Iva = p.Plato1.Iva;
                    nuevoPlato.Disponible = p.Plato1.Disponible;
                    listaPlatos.Add(nuevoPlato);
                }
                return listaPlatos;
            }
            catch (NullReferenceException)
            {
                throw;
            }
            catch (SystemException)
            {
                throw;
            }
            catch (Exception)
            {
                throw;
            }
        }

        //mètode per eliminar els plats que fomen part de la carta passada com a paràmetre
        public void EliminarPlatosCarta(int idCarta)
        {
            try
            {
                //obrim la connexió
                ConectarBBDD();
                var platos = (from p in db.PlatosCarta
                              where (p.Carta == idCarta)
                              select p);
                foreach (var p in platos)
                {
                    db.PlatosCarta.DeleteOnSubmit(p);
                    db.SubmitChanges();
                    DesconectarBBDD();
                }
            }
            catch (NullReferenceException)
            {
                throw;
            }
            catch (SystemException)
            {
                throw;
            }
            catch (Exception)
            {
                throw;
            }
        }

        //mètode per eliminar una carta i els seus plats. Al eliminar la carta els plats de la 
        //taula PlatosCarta s'eliminen en cascada
        public void EliminarCarta(int idCarta)
        {
            try
            {
                ConectarBBDD();
                Carta carta = db.Carta.Single(c => c.Id == idCarta);

                if (carta != null)
                {
                    db.Carta.DeleteOnSubmit(carta);
                    db.SubmitChanges();
                    DesconectarBBDD();
                }
                else
                {
                    DesconectarBBDD();
                }
            }
            catch (NullReferenceException)
            {
                throw;
            }
            catch (SystemException)
            {
                throw;
            }
            catch (Exception)
            {
                throw;
            }
        }

        //mètode per activar l'atribut boleà "publicar" de la carta passada per paràmetre
        public void PublicarCarta(int idCarta)
        {
            try
            {
                ConectarBBDD();
                Carta carta = db.Carta.Single(c => c.Id == idCarta);

                if (carta != null)
                {
                    carta.Publicar = true;
                    db.SubmitChanges();
                    DesconectarBBDD();
                }
                else
                {
                    DesconectarBBDD();
                }
            }

            catch (NullReferenceException)
            {
                throw;
            }
            catch (SystemException)
            {
                throw;
            }
            catch (Exception)
            {
                throw;
            }
        }

        //mètode per modificar l'atibut publicar de les cartes
        public void DespublicarCarta()
        {
            try
            {
                ConectarBBDD();
                List<Carta> listaCartas = new List<Carta>();
                //obrim la connexió
                ConectarBBDD();
                var cartas = (from c in db.Carta select c);

                foreach (var c in cartas)
                {
                    c.Publicar = false;
                    db.SubmitChanges();
                    DesconectarBBDD();
                }
                    DesconectarBBDD();
            }
            catch (NullReferenceException)
            {
                throw;
            }
            catch (SystemException)
            {
                throw;
            }
            catch (Exception)
            {
                throw;
            }
        }



        //mètode per llistar els plats que fomen part de la carta que s'ha de visualitzar al web, per seccio
        public List<Plato> ImprimirCartaWeb(string idSeccion)
        {
            try
            {
                List<Plato> listaPlatos = new List<Plato>();
                //obrim la connexió
                ConectarBBDD();
                var platos = (from p in db.PlatosCarta
                              where (p.Carta1.Publicar == true && p.Plato1.Seccion1.nombre == idSeccion)
                              orderby p.Plato1.Idioma1
                              select p);
                //tanquem la connexió
                DesconectarBBDD();

                foreach (var p in platos)
                {
                    Plato nuevoPlato = new Plato();
                    nuevoPlato.Id = p.Plato1.Id;
                    nuevoPlato.Idioma1 = p.Plato1.Idioma1;
                    nuevoPlato.Idioma2 = p.Plato1.Idioma2;
                    nuevoPlato.Idioma3 = p.Plato1.Idioma3;
                    nuevoPlato.Precio = p.Plato1.Precio;
                    nuevoPlato.Seccion = p.Plato1.Seccion;
                    nuevoPlato.MinComensales = p.Plato1.MinComensales;
                    nuevoPlato.Iva = p.Plato1.Iva;
                    nuevoPlato.Disponible = p.Plato1.Disponible;
                    listaPlatos.Add(nuevoPlato);
                }
                return listaPlatos;
            }
            catch (NullReferenceException)
            {
                throw;
            }
            catch (SystemException)
            {
                throw;
            }
            catch (Exception)
            {
                throw;
            }
        }

        //********* Mètodes per gestionar els CLIENTS ************

        //mètode que obtè una llista de clients
        public List<Cliente> ListarClientes()
        {
            try
            {
                //obrim la connexió
                ConectarBBDD();
                var clientes = (from c in db.Cliente orderby c.Nombre select c).ToList();
                //tanquem la connexió
                DesconectarBBDD();
                return clientes;
            }
            catch (NullReferenceException)
            {
                throw;
            }
            catch (SystemException)
            {
                throw;
            }
            catch (Exception)
            {
                throw;
            }
        }

        //mètode que obtè la llista de clients amb nom igual al passat com a paràmetre
        public List<Cliente> ListarClientesNombre(String pNombre)
        {
            try
            {
                //obrim la connexió
                ConectarBBDD();
                var clientes = (from c in db.Cliente
                              where (c.Nombre.Contains(pNombre))
                              orderby c.Nombre
                              select c).ToList();
                //tanquem la connexió
                DesconectarBBDD();
                return clientes;
            }
            catch (NullReferenceException)
            {
                throw;
            }
            catch (SystemException)
            {
                throw;
            }
            catch (Exception)
            {
                throw;
            }
        }


        //mètode per afegir un client i que retorna el client creat
        public Cliente AltaCliente(string pNombre, string pMail, string pPw, string pTelef, bool pMenu, bool pNews)
        {
            try
            {
                ConectarBBDD();

                Cliente c = new Cliente();
                c.Nombre = pNombre;
                c.Mail = pMail;
                c.Password = pPw;
                c.Telefono = pTelef;
                c.SusMenu = pMenu;
                c.SusOfertas = pNews;

                db.Cliente.InsertOnSubmit(c);
                db.SubmitChanges();

                DesconectarBBDD();
                return c;
            }
            catch (NullReferenceException)
            {
                throw;
            }
            catch (SystemException)
            {
                throw;
            }
            catch (Exception)
            {
                throw;
            }
        }



        //mètode per modificar un client
        public void ModificarCliente(int idCliente, string pNombre, string pMail, string pPw, string pTelef, bool pMenu, bool pNews)
        {
            try
            {
                ConectarBBDD();
                Cliente client = db.Cliente.Single(c => c.Id == idCliente);
                if (client != null)
                {
                    client.Nombre = pNombre;
                    client.Mail = pMail;
                    client.Password = pPw;
                    client.Telefono = pTelef;
                    client.SusMenu = pMenu;
                    client.SusOfertas = pNews;
                    db.SubmitChanges();
                    DesconectarBBDD();
                }
                else
                {
                    DesconectarBBDD();
                }
            }
            catch (NullReferenceException)
            {
                throw;
            }
            catch (SystemException)
            {
                throw;
            }
            catch (Exception)
            {
                throw;
            }
        }


        //mètode per buscar el username d'un client
        public Boolean buscarUsername(String pMail)
        {
            try
            {
                ConectarBBDD();
                //Consulta si existeix un client amb mail igual al rebut com a paràmetre.
                var result = from c in db.Cliente
                             where c.Mail == pMail 
                             select c;
                if (result.Count() != 0) //Si existeix algun registre es que hem trobat el client.
                {
                    DesconectarBBDD();
                    return true;
                }
                else // no s'ha trobat el client amb aquest mail
                {
                    DesconectarBBDD();
                    return false;
                }
            }
            catch (NullReferenceException)
            {
                throw;
            }
            catch (SystemException)
            {
                throw;
            }
            catch (Exception)
            {
                throw;
            }
        }

        //mètode per validar l'username i password d'un client
        public Boolean validarUsername(String pMail, string pPw)
        {
            try
            {
                ConectarBBDD();
                //Consulta si existeix un client amb mail i password igual al rebut com a paràmetre.
                var result = from c in db.Cliente
                             where c.Mail == pMail && c.Password == pPw
                             select c; 
                if (result.Count() != 0) //Si existeix algun registre es que hem trobat el client.
                {
                    DesconectarBBDD();
                    return true;
                }
                else // no s'ha trobat el client amb aquest mail
                {
                    DesconectarBBDD();
                    return false;
                }
            }
            catch (NullReferenceException)
            {
                throw;
            }
            catch (SystemException)
            {
                throw;
            }
            catch (Exception)
            {
                throw;
            }
        }


         
        //mètode que retorna un client
        public Cliente BuscarCliente(String pMail, string pPw)
        {
            try
            {
                ConectarBBDD();
                //Consulta si existeix un client amb username i password iguals als rebuts com a paràmetre.
                Cliente client = db.Cliente.Single(c => c.Mail == pMail && c.Password == pPw);
                if (client != null)
                {
                    Cliente nuevoCliente = new Cliente();
                    DesconectarBBDD();
                    nuevoCliente.Id = client.Id;
                    nuevoCliente.Nombre = client.Nombre;
                    nuevoCliente.Telefono = client.Telefono;
                    nuevoCliente.Mail = client.Mail;
                    nuevoCliente.Password = client.Password;
                    nuevoCliente.SusMenu = client.SusMenu;
                    nuevoCliente.SusOfertas = client.SusOfertas;
                    return nuevoCliente;
                }
                else // no s'ha trobat el client
                {
                    DesconectarBBDD();
                    return null;
                }
            }
            catch (NullReferenceException)
            {
                throw;
            }
            catch (SystemException)
            {
                throw;
            }
            catch (Exception)
            {
                throw;
            }
        }


        //mètode que obtè la llista de mails de la llista de distribució a notícies
        public List<String> ListaDistribucionNoticias()
        {
            try
            {
                List<String> listaDistribucion = new List<String>();
                //obrim la connexió
                ConectarBBDD();
                var result = (from c in db.Cliente where c.SusOfertas==true select c).ToList();
                //tanquem la connexió
                DesconectarBBDD();
                foreach (var r in result)
                {
                    string mail = r.Mail;
                    listaDistribucion.Add(mail);
                }
                
                return listaDistribucion;
            }
            catch (NullReferenceException)
            {
                throw;
            }
            catch (SystemException)
            {
                throw;
            }
            catch (Exception)
            {
                throw;
            }
        }

        //mètode que obtè la llista de mails de la llista de distribució a menus
        public List<String> ListaDistribucionMenus()
        {
            try
            {
                List<String> listaDistribucion = new List<String>();

                //obrim la connexió
                ConectarBBDD();
                var result = (from c in db.Cliente where c.SusMenu == true select c).ToList();
                //tanquem la connexió
                DesconectarBBDD();
                foreach (var r in result)
                {
                    string mail = r.Mail;
                    listaDistribucion.Add(mail);
                }
                return listaDistribucion;
            }
            catch (NullReferenceException)
            {
                throw;
            }
            catch (SystemException)
            {
                throw;
            }
            catch (Exception)
            {
                throw;
            }
        }

        //********* Mètodes per gestionar les RESERVES ************

        //mètode per llistar les reserves d'una data
        public List<Reserva> ListarReservasFecha(DateTime pFecha)
        {
            try
            {
                List<Reserva> listaReservas = new List<Reserva>();

                //obrim la connexió
                ConectarBBDD();
                var result = (from r in db.Reserva
                              where (r.Fecha == pFecha)
                              orderby r.Fecha
                              select r);
                //tanquem la connexió
                DesconectarBBDD();

                foreach (var r in result)
                {
                    Reserva nuevaReserva = new Reserva();
                    nuevaReserva.id = r.id;
                    nuevaReserva.Cliente = r.Cliente;
                    nuevaReserva.Nombre = r.Nombre;
                    nuevaReserva.Telefono = r.Telefono;
                    nuevaReserva.Mail = r.Mail;
                    nuevaReserva.Comensales = r.Comensales;
                    nuevaReserva.Estado = r.Estado;
                    nuevaReserva.Fuente = r.Fuente;
                    nuevaReserva.Fecha = r.Fecha;
                    nuevaReserva.Mesa = r.Mesa;
                    nuevaReserva.Turno = r.Turno;
                    listaReservas.Add(nuevaReserva);
                }
                return listaReservas;
            }
            catch (NullReferenceException)
            {
                throw;
            }
            catch (SystemException)
            {
                throw;
            }
            catch (Exception)
            {
                throw;
            }
        }

        //mètode per llistar les reserves amb nº ID
        public List<Reserva> ListarReservasNumero(int idReserva)
        {
            try
            {
                List<Reserva> listaReservas = new List<Reserva>();

                //obrim la connexió
                ConectarBBDD();
                var result = (from r in db.Reserva
                              where (r.id == idReserva)
                              select r);
                //tanquem la connexió
                DesconectarBBDD();

                foreach (var r in result)
                {
                    Reserva nuevaReserva = new Reserva();
                    nuevaReserva.id = r.id;
                    nuevaReserva.Cliente = r.Cliente;
                    nuevaReserva.Nombre = r.Nombre;
                    nuevaReserva.Telefono = r.Telefono;
                    nuevaReserva.Mail = r.Mail;
                    nuevaReserva.Comensales = r.Comensales;
                    nuevaReserva.Estado = r.Estado;
                    nuevaReserva.Fuente = r.Fuente;
                    nuevaReserva.Fecha = r.Fecha;
                    nuevaReserva.Mesa = r.Mesa;
                    nuevaReserva.Turno = r.Turno;
                    listaReservas.Add(nuevaReserva);
                }
                return listaReservas;
            }
            catch (NullReferenceException)
            {
                throw;
            }
            catch (SystemException)
            {
                throw;
            }
            catch (Exception)
            {
                throw;
            }
        }

        

        //mètode per obtenir les resreves d'un client
        public List<Reserva> ListarReservasCliente(int idCliente)
        {
            try
            {
                List<Reserva> listaReservas = new List<Reserva>();

                //obrim la connexió
                ConectarBBDD();
                var result = (from r in db.Reserva
                              where (r.Cliente == idCliente)
                              orderby r.Fecha descending
                              select r);
                //tanquem la connexió
                DesconectarBBDD();

                foreach (var r in result)
                {
                    Reserva nuevaReserva = new Reserva();
                    nuevaReserva.id = r.id;
                    nuevaReserva.Cliente = r.Cliente;
                    nuevaReserva.Nombre = r.Nombre;
                    nuevaReserva.Telefono = r.Telefono;
                    nuevaReserva.Mail = r.Mail;
                    nuevaReserva.Comensales = r.Comensales;
                    nuevaReserva.Estado = r.Estado;
                    nuevaReserva.Fuente = r.Fuente;
                    nuevaReserva.Fecha = r.Fecha;
                    nuevaReserva.Mesa = r.Mesa;
                    nuevaReserva.Turno = r.Turno;
                    listaReservas.Add(nuevaReserva);
                }
                return listaReservas;
            }
            catch (NullReferenceException)
            {
                throw;
            }
            catch (SystemException)
            {
                throw;
            }
            catch (Exception)
            {
                throw;
            }
        }

        

        //mètode per afegir una reserva i que retorna el nº de reserva
        public int AltaReserva(int pCli, string pNombre, string pTelef, string pMail, int pComens, string pEstado, string pFuente, DateTime pFecha, int pMesa, String pTurno)
        {
            try
            {
                ConectarBBDD();

                Reserva r = new Reserva();
                r.Cliente = pCli;
                r.Nombre = pNombre;
                r.Telefono = pTelef;
                r.Mail = pMail;
                r.Comensales = pComens;
                r.Estado = pEstado;
                r.Fuente = pFuente;
                r.Fecha = pFecha;
                r.Mesa = pMesa;
                r.Turno = pTurno;

                AltaAforo(pFecha, pMesa, pTurno);//alta de l'aforament
                db.Reserva.InsertOnSubmit(r);//alta de la reserva
                db.SubmitChanges();

                DesconectarBBDD();
                return r.id;
            }
            catch (NullReferenceException)
            {
                throw;
            }
            catch (SystemException)
            {
                throw;
            }
            catch (Exception)
            {
                throw;
            }
        }

        //mètode per eliminar una reserva i l'aforament associat a ella
        public void EliminarReserva(int idReserva, DateTime pFecha, int pMesa, string pTurno)
        {
            try
            {
                ConectarBBDD();
                Reserva reserva = db.Reserva.Single(r => r.id == idReserva);

                if (reserva != null)
                {
                    //eliminar reserva
                    db.Reserva.DeleteOnSubmit(reserva);
                    db.SubmitChanges();
                    //eliminar aforament
                    EliminarAforo(pFecha, pMesa, pTurno);
                    DesconectarBBDD();
                }
                else
                {
                    DesconectarBBDD();
                }
            }
            catch (NullReferenceException)
            {
                throw;
            }
            catch (SystemException)
            {
                throw;
            }
            catch (Exception)
            {
                throw;
            }
        }



        


        //********* Mètodes per gestionar les OFERTES ************

        public List<Oferta> ListarOfertas()
        {
            List<Oferta> listaOfertas = new List<Oferta>();
            try
            {
                //obrim la connexió
                ConectarBBDD();
                var query = (from o in db.Oferta orderby o.Fecha descending select o);
                //tanquem la connexió
                DesconectarBBDD();

                foreach (var o in query)
                {
                    Oferta nuevaOferta = new Oferta();
                    nuevaOferta.Id = o.Id;
                    nuevaOferta.Fecha = o.Fecha;
                    nuevaOferta.Nombre= o.Nombre;
                    nuevaOferta.Contenido = o.Contenido;
                    listaOfertas.Add(nuevaOferta);
                }
                return listaOfertas;
            }
            catch (NullReferenceException)
            {
                throw;
            }
            catch (SystemException)
            {
                throw;
            }
            catch (Exception)
            {
                throw;
            }

        }



        //mètode per afegir una nova oferta
        public void AñadirOferta(string pNombre, string pContent, DateTime pFecha)
        {
            try
            {
                ConectarBBDD();//obrim la connexió
                Oferta o = new Oferta();
                o.Nombre = pNombre;
                o.Contenido = pContent;
                o.Fecha = pFecha;
                db.Oferta.InsertOnSubmit(o);
                db.SubmitChanges();
                DesconectarBBDD();//tanquem la connexió
            }
            catch (NullReferenceException)
            {
                throw;
            }
            catch (SystemException)
            {
                throw;
            }
            catch (Exception)
            {
                throw;
            }
        }

        //mètode per modificar una oferta
        public void ModificarOferta(int pId, string newNombre, string newContent, DateTime newFecha)
        {
            try
            {
                ConectarBBDD();
                Oferta oferta = db.Oferta.Single(o => o.Id == pId);

                if (oferta != null)
                {
                    oferta.Nombre = newNombre;
                    oferta.Contenido = newContent;
                    oferta.Fecha = newFecha;
                    db.SubmitChanges();
                    DesconectarBBDD();
                }
                else
                {
                    DesconectarBBDD();
                }
            }
            catch (NullReferenceException)
            {
                throw;
            }
            catch (SystemException)
            {
                throw;
            }
            catch (Exception)
            {
                throw;
            }
        }


       

        //********* Mètodes per gestionar l'AFORAMENT ************

        //mètode per afegir aforament
        public void AltaAforo(DateTime pFecha, int pMesa, String pTurno)
        {
            try
            {
                ConectarBBDD();//obrim la connexió
                Aforo a = new Aforo();
                a.Fecha = pFecha;
                a.Mesa = pMesa;
                a.Turno = pTurno;
                db.Aforo.InsertOnSubmit(a);
                db.SubmitChanges();
                DesconectarBBDD();//tanquem la connexió
            }
            catch (NullReferenceException)
            {
                throw;
            }
            catch (SystemException)
            {
                throw;
            }
            catch (Exception)
            {
                throw;
            }
        }

        //mètode per eliminar aforament
        public void EliminarAforo(DateTime pFecha, int pMesa, String pTurno)
        {
            try
            {
                ConectarBBDD();
                Aforo aforo = db.Aforo.Single(a => a.Fecha == pFecha && a.Mesa == pMesa && a.Turno == pTurno);

                if (aforo != null)
                {
                    db.Aforo.DeleteOnSubmit(aforo);
                    db.SubmitChanges();
                    DesconectarBBDD();
                }
                else
                {
                    DesconectarBBDD();
                }
            }
            catch (NullReferenceException)
            {
                throw;
            }
            catch (SystemException)
            {
                throw;
            }
            catch (Exception)
            {
                throw;
            }
        }



        

        #endregion        
    }
}
